@@ -146,6 +146,7 @@ return {
                 IPA.user.stageuser_sidebar_policy
             ],
             deleter_dialog: {
+                title: '@i18n:objects.user.remove',
                 $factory: IPA.user.deleter_dialog
             }
         },
@@ -192,7 +193,8 @@ return {
                             name: 'krbprincipalname',
                             item_name: 'principal',
                             child_spec: {
-                                $type: 'krb_principal'
+                                $type: 'non_editable_row',
+                                data_name: 'krb-principal'
                             }
                         },
                         {
@@ -216,6 +218,18 @@ return {
                             },
                             label: '@i18n:objects.cert.certificates'
                         },
+                        {
+                            $type: 'certmap_multivalued',
+                            name: 'ipacertmapdata',
+                            item_name: 'certmapdata',
+                            child_spec: {
+                                $type: 'non_editable_row',
+                                data_name: 'certmap'
+                            },
+                            tooltip: {
+                                title: '@mc:user_add_certmapdata.doc'
+                            }
+                        },
                         {
                             $type: 'checkboxes',
                             name: 'ipauserauthtype',
@@ -347,6 +361,42 @@ return {
                     fields: [
                         { $type: 'multivalued', name: 'carlicense' }
                     ]
+                },
+                {
+                    name: 'smb_attributes',
+                    label: '@i18n:objects.smb_attributes.title',
+                    show_cond: ['oc_ipantuserattrs'],
+                    fields: [{
+                            name: 'ipantlogonscript',
+                            tooltip: {
+                                title: '@i18n:objects.smb_attributes.ipantlogonscript_tooltip'
+                            }
+                        },
+                        {
+                            name: 'ipantprofilepath',
+                            tooltip: {
+                                title: '@i18n:objects.smb_attributes.ipantprofilepath_tooltip'
+                            }
+                        },
+                        {
+                            name: 'ipanthomedirectory',
+                            tooltip: {
+                                title: '@i18n:objects.smb_attributes.ipanthomedirectory_tooltip'
+                            }
+                        },
+                        {
+                            name: 'ipanthomedirectorydrive',
+                            $type: 'select',
+                            options: IPA.create_options([
+                                'A:', 'B:', 'C:', 'D:', 'E:', 'F:', 'G:', 'H:', 'I:',
+                                'J:', 'K:', 'L:', 'M:', 'N:', 'O:', 'P:', 'Q:', 'R:',
+                                'S:', 'T:', 'U:', 'V:', 'W:', 'X:', 'Y:', 'Z:'
+                            ]),
+                            tooltip: {
+                                title: '@i18n:objects.smb_attributes.ipanthomedirectorydrive_tooltip'
+                            }
+                        }
+                    ]
                 }
             ],
             actions: [
@@ -393,6 +443,7 @@ return {
                     label: '@i18n:objects.user.unlock',
                     needs_confirm: true,
                     hide_cond: ['preserved-user'],
+                    disable_cond: ['no-password'],
                     confirm_msg: '@i18n:objects.user.unlock_confirm'
                 },
                 {
@@ -403,7 +454,8 @@ return {
                 },
                 {
                     $type: 'cert_request',
-                    hide_cond: ['preserved-user']
+                    hide_cond: ['preserved-user'],
+                    title: '@i18n:objects.cert.issue_for_user'
                 }
             ],
             header_actions: [
@@ -427,6 +479,8 @@ return {
                     },
                     IPA.user.self_service_other_user_evaluator,
                     IPA.user.preserved_user_evaluator,
+                    IPA.user.no_password_evaluator,
+                    IPA.object_class_evaluator,
                     IPA.cert.certificate_evaluator
                 ],
                 summary_conditions: [
@@ -449,19 +503,25 @@ return {
             $type: 'association',
             $pre_ops: [ IPA.user.association_facet_ss_pre_op ],
             name: 'memberof_group',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.user.add_into_groups',
+            remove_title: '@i18n:objects.user.remove_from_groups'
         },
         {
             $type: 'association',
             $pre_ops: [ IPA.user.association_facet_ss_pre_op ],
             name: 'memberof_netgroup',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.user.add_into_netgroups',
+            remove_title: '@i18n:objects.user.remove_from_netgroups'
         },
         {
             $type: 'association',
             $pre_ops: [ IPA.user.association_facet_ss_pre_op ],
             name: 'memberof_role',
-            associator: IPA.serial_associator
+            associator: IPA.serial_associator,
+            add_title: '@i18n:objects.user.add_into_roles',
+            remove_title: '@i18n:objects.user.remove_from_roles'
         },
         {
             $type: 'association',
@@ -469,7 +529,9 @@ return {
             name: 'memberof_hbacrule',
             associator: IPA.serial_associator,
             add_method: 'add_user',
-            remove_method: 'remove_user'
+            add_title: '@i18n:objects.user.add_into_hbac',
+            remove_method: 'remove_user',
+            remove_title: '@i18n:objects.user.remove_from_hbac'
         },
         {
             $type: 'association',
@@ -477,13 +539,16 @@ return {
             name: 'memberof_sudorule',
             associator: IPA.serial_associator,
             add_method: 'add_user',
-            remove_method: 'remove_user'
+            add_title: '@i18n:objects.user.add_into_sudo',
+            remove_method: 'remove_user',
+            remove_title: '@i18n:objects.user.remove_from_sudo'
         }
     ],
     standard_association_facets: {
         $pre_ops: [ IPA.user.association_facet_ss_pre_op ]
     },
     adder_dialog: {
+        title: '@i18n:objects.user.add',
         $factory: IPA.user.adder_dialog,
         sections: [
             {
@@ -548,6 +613,7 @@ IPA.user.details_facet = function(spec, no_init) {
         });
 
         var user_command = that.details_facet_create_refresh_command();
+
         batch.add_command(user_command);
 
         var pwpolicy_command = rpc.command({
@@ -598,6 +664,7 @@ IPA.user.details_facet = function(spec, no_init) {
             retry: false,
             options: {
                 user: [ pkey ],
+                sizelimit: 0,
                 all: true
             }
         });
@@ -607,6 +674,11 @@ IPA.user.details_facet = function(spec, no_init) {
         return batch;
     };
 
+    that.update_on_success = function(data, text_status, xhr) {
+        that.on_update.notify();
+        that.nofify_update_success();
+        that.refresh();
+    };
 
     if (!no_init) that.init_details_facet();
 
@@ -721,7 +793,7 @@ IPA.user.password_dialog = function(spec) {
     var that = dialogs.command_dialog(spec);
 
     that.is_self_service = function() {
-        var self_service = that.args[0] === IPA.whoami.uid[0];
+        var self_service = that.args[0] === IPA.whoami.data.uid[0];
         return self_service;
     };
 
@@ -821,7 +893,7 @@ IPA.user.add_otptoken_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'add_otptoken';
-    spec.label = spec.label || '@i18n:objects.otptoken.add_token';
+    spec.label = spec.label || '@i18n:objects.otptoken.add';
     spec.disable_cond = spec.disable_cond || ['self-service-other'];
 
     var that = IPA.action(spec);
@@ -881,7 +953,8 @@ IPA.user.self_service_other_user_evaluator = function(spec) {
         that.state = [];
 
         var value = that.adapter.load(data);
-        if (IPA.is_selfservice && IPA.whoami.uid[0] !== value[0]) {
+        if (IPA.is_aduser_selfservice ||
+            (IPA.is_selfservice && IPA.whoami.data.uid[0] !== value[0])) {
             that.state.push('self-service-other');
         }
 
@@ -1045,6 +1118,35 @@ IPA.user.deleter_dialog = function(spec) {
     return that;
 };
 
+IPA.user.no_password_evaluator = function(spec) {
+
+    spec = spec || {};
+    spec.event = spec.event || 'post_load';
+
+    var that = IPA.state_evaluator(spec);
+    that.name = spec.name || 'no_password_evaluator';
+    that.param = spec.param || 'has_password';
+    that.adapter = builder.build('adapter', { $type: 'adapter'}, { context: that });
+
+    /**
+     * Evaluates if user has no password
+     */
+    that.on_event = function(data) {
+
+        var old_state = that.state;
+        that.state = [];
+
+        var has_password = that.adapter.load(data)[0];
+        if (!has_password) {
+            that.state.push('no-password');
+        }
+
+        that.notify_on_change(old_state);
+    };
+
+    return that;
+};
+
 exp.entity_spec = make_spec();
 exp.register = function() {
     var e = reg.entity;
