@@ -51,6 +51,7 @@ from samba.dcerpc import security, lsa, drsblobs, nbt, netlogon
 from samba.ndr import ndr_pack, ndr_print
 from samba import net
 from samba import arcfour_encrypt
+from samba import ntstatus
 import samba
 
 import ldap as _ldap
@@ -192,7 +193,7 @@ class ExtendedDNControl(LDAPControl):
         )
 
 
-class DomainValidator(object):
+class DomainValidator:
     ATTR_FLATNAME = 'ipantflatname'
     ATTR_SID = 'ipantsecurityidentifier'
     ATTR_TRUSTED_SID = 'ipanttrusteddomainsid'
@@ -723,9 +724,10 @@ class DomainValidator(object):
                 entries = None
 
                 try:
-                    ldap_uri = ipaldap.get_ldap_uri(host)
-                    conn = ipaldap.LDAPClient(
-                        ldap_uri,
+                    # AD does not support SASL + TLS at the same time
+                    # https://msdn.microsoft.com/en-us/library/cc223500.aspx
+                    conn = ipaldap.LDAPClient.from_hostname_plain(
+                        host,
                         no_schema=True,
                         decode_attrs=False
                     )
@@ -823,7 +825,7 @@ def string_to_array(what):
     return [ord(v) for v in what]
 
 
-class TrustDomainInstance(object):
+class TrustDomainInstance:
 
     def __init__(self, hostname, creds=None):
         self.parm = param.LoadParm()
@@ -1105,6 +1107,25 @@ class TrustDomainInstance(object):
         original forest.
         """
 
+        def domain_name_from_ftinfo(ftinfo):
+            """
+            Returns a domain name string from a ForestTrustRecord
+
+            :param ftinfo: LSA ForestTrustRecord to parse
+            """
+            if ftinfo.type == lsa.LSA_FOREST_TRUST_DOMAIN_INFO:
+                return ftinfo.forest_trust_data.dns_domain_name.string
+            elif ftinfo.type == lsa.LSA_FOREST_TRUST_TOP_LEVEL_NAME:
+                return ftinfo.forest_trust_data.string
+            elif ftinfo.type == lsa.LSA_FOREST_TRUST_TOP_LEVEL_NAME_EX:
+                # We should ignore TLN exclusion record because it
+                # is already an exclusion so we aren't going to
+                # change anything here
+                return None
+            else:
+                # Ignore binary blobs we don't know about
+                return None
+
         # List of entries for unsolved conflicts
         result = []
 
@@ -1145,18 +1166,26 @@ class TrustDomainInstance(object):
                     e1.time = e.time
                     e1.forest_trust_data = e.forest_trust_data
 
+                    # We either have a domain struct, a TLN name,
+                    # or a TLN exclusion name in the list.
+                    # The rest we should skip, those are binary blobs
+                    dns_domain_name = domain_name_from_ftinfo(e)
+
                     # Search for a match in the topology of another domain
                     # if there is a match, we have to convert a record
                     # into a TLN exclusion to allow its routing to the
                     # another domain
                     for r in another_domain.ftinfo_records:
-                        if r['rec_name'] == e.forest_trust_data.string:
+                        # r['rec_name'] cannot be None, thus we can ignore
+                        # the case when dns_domain_name is None
+                        if r['rec_name'] == dns_domain_name:
                             is_our_record = True
 
                             # Convert e1 into an exclusion record
                             e1.type = lsa.LSA_FOREST_TRUST_TOP_LEVEL_NAME_EX
                             e1.flags = 0
                             e1.time = trust_timestamp
+                            e1.forest_trust_data.string = dns_domain_name
                             break
                     entries.append(e1)
 
@@ -1180,11 +1209,29 @@ class TrustDomainInstance(object):
                 # Update the forest trust information now
                 ldname = lsa.StringLarge()
                 ldname.string = rec.name.string
-                cninfo = self._pipe.lsaRSetForestTrustInformation(
-                             self._policy_handle,
-                             ldname,
-                             lsa.LSA_FOREST_TRUST_DOMAIN_INFO,
-                             fti, 0)
+                cninfo = None
+                try:
+                    cninfo = self._pipe.lsaRSetForestTrustInformation(
+                        self._policy_handle,
+                        ldname,
+                        lsa.LSA_FOREST_TRUST_DOMAIN_INFO,
+                        fti, 0)
+                except samba.NTSTATUSError as error:
+                    # Handle NT_STATUS_INVALID_PARAMETER separately
+                    if ntstatus.NT_STATUS_INVALID_PARAMETER == error.args[0]:
+                        result.append(rec)
+                        logger.error("Unable to resolve conflict for "
+                                     "DNS domain %s in the forest %s "
+                                     "for in-forest domain %s. Trust cannot "
+                                     "be established unless this conflict "
+                                     "is fixed manually.",
+                                     another_domain.info['dns_domain'],
+                                     self.info['dns_domain'],
+                                     rec.name.string)
+                    else:
+                        raise assess_dcerpc_error(error)
+
+
                 if cninfo:
                     result.append(rec)
                     logger.error("When defining exception for DNS "
@@ -1213,9 +1260,9 @@ class TrustDomainInstance(object):
         # Otherwise, raise TrustTopologyConflictError() exception
         domains = [x.name.string for x in result]
         raise errors.TrustTopologyConflictError(
-                              target=self.info['dns_domain'],
-                              conflict=another_domain.info['dns_domain'],
-                              domains=domains)
+            forest=self.info['dns_domain'],
+            conflict=another_domain.info['dns_domain'],
+            domains=domains)
 
 
 
@@ -1568,7 +1615,7 @@ def retrieve_remote_domain(hostname, local_flatname,
     return rd
 
 
-class TrustDomainJoins(object):
+class TrustDomainJoins:
     def __init__(self, api):
         self.api = api
         self.local_domain = None
