@@ -19,6 +19,7 @@
 
 from __future__ import absolute_import
 
+import configparser
 import logging
 import os
 import stat
@@ -34,7 +35,6 @@ import time
 import datetime
 
 import six
-from six.moves import configparser
 
 from ipalib.install import certmonger, sysrestore
 from ipapython import dogtag
@@ -136,7 +136,7 @@ def is_ipa_issued_cert(api, cert):
     return DN(cert.issuer) == cacert_subject
 
 
-class CertDB(object):
+class CertDB:
     """An IPA-server-specific wrapper around NSS
 
     This class knows IPA-specific details such as nssdir location, or the
@@ -377,14 +377,16 @@ class CertDB(object):
         except ipautil.CalledProcessError:
             return None
 
-    def track_server_cert(self, nickname, principal, password_file=None, command=None):
+    def track_server_cert(
+            self, nickname, principal,
+            password_file=None, command=None, profile=None):
         """
         Tell certmonger to track the given certificate nickname.
         """
         try:
             request_id = certmonger.start_tracking(
                 self.secdir, nickname=nickname, pinfile=password_file,
-                post_command=command)
+                post_command=command, profile=profile)
         except RuntimeError as e:
             logger.error("certmonger failed starting to track certificate: %s",
                          str(e))
@@ -711,7 +713,7 @@ class CertDB(object):
         self.nssdb.convert_db()
 
 
-class _CrossProcessLock(object):
+class _CrossProcessLock:
     _DATETIME_FORMAT = '%Y%m%d%H%M%S%f'
 
     def __init__(self, filename):
