@@ -104,6 +104,8 @@ class HTTPInstance(service.Service):
             AUTOREDIR='' if auto_redirect else '#',
             CRL_PUBLISH_PATH=paths.PKI_CA_PUBLISH_DIR,
             FONTS_DIR=paths.FONTS_DIR,
+            FONTS_OPENSANS_DIR=paths.FONTS_OPENSANS_DIR,
+            FONTS_FONTAWESOME_DIR=paths.FONTS_FONTAWESOME_DIR,
             GSSAPI_SESSION_KEY=paths.GSSAPI_SESSION_KEY,
             IPA_CUSTODIA_SOCKET=paths.IPA_CUSTODIA_SOCKET,
             IPA_CCACHES=paths.IPA_CCACHES,
@@ -121,7 +123,7 @@ class HTTPInstance(service.Service):
         self.step("disabling nss.conf", self.disable_nss_conf)
         self.step("configuring mod_ssl certificate paths",
                   self.configure_mod_ssl_certs)
-        self.step("setting mod_ssl protocol list to TLSv1.0 - TLSv1.2",
+        self.step("setting mod_ssl protocol list",
                   self.set_mod_ssl_protocol)
         self.step("configuring mod_ssl log directory",
                   self.set_mod_ssl_logdir)
@@ -169,7 +171,7 @@ class HTTPInstance(service.Service):
         # Clean up existing ccaches
         # Make sure that empty env is passed to avoid passing KRB5CCNAME from
         # current env
-        installutils.remove_file(paths.HTTP_CCACHE)
+        ipautil.remove_file(paths.HTTP_CCACHE)
         for f in os.listdir(paths.IPA_CCACHES):
             os.remove(os.path.join(paths.IPA_CCACHES, f))
 
@@ -226,7 +228,9 @@ class HTTPInstance(service.Service):
 
     def disable_nss_conf(self):
         """
-        Backs up and removes the original nss.conf file.
+        Backs up the original nss.conf file and replace it with the empty one.
+        Empty file avoids recreation of nss.conf in case the package is
+        reinstalled.
 
         There is no safe way to co-exist since there is no safe port
         to make mod_nss use, disable it completely.
@@ -236,12 +240,11 @@ class HTTPInstance(service.Service):
             # (mod_nss -> mod_ssl upgrade scenario)
             if not self.fstore.has_file(paths.HTTPD_NSS_CONF):
                 self.fstore.backup_file(paths.HTTPD_NSS_CONF)
-            installutils.remove_file(paths.HTTPD_NSS_CONF)
+
+        open(paths.HTTPD_NSS_CONF, 'w').close()
 
     def set_mod_ssl_protocol(self):
-        directivesetter.set_directive(paths.HTTPD_SSL_CONF,
-                                   'SSLProtocol',
-                                   '+TLSv1 +TLSv1.1 +TLSv1.2', False)
+        tasks.configure_httpd_protocol()
 
     def set_mod_ssl_logdir(self):
         tasks.setup_httpd_logging()
@@ -524,7 +527,7 @@ class HTTPInstance(service.Service):
                 logger.debug("%s", error)
 
         # Remove the configuration files we create
-        installutils.remove_keytab(self.keytab)
+        ipautil.remove_keytab(self.keytab)
         remove_files = [
             paths.HTTP_CCACHE,
             paths.HTTPD_CERT_FILE,
@@ -548,7 +551,7 @@ class HTTPInstance(service.Service):
             remove_files.append(paths.HTTPD_IPA_WSGI_MODULES_CONF)
 
         for filename in remove_files:
-            installutils.remove_file(filename)
+            ipautil.remove_file(filename)
 
         try:
             os.rmdir(paths.SYSTEMD_SYSTEM_HTTPD_D_DIR)
@@ -582,11 +585,14 @@ class HTTPInstance(service.Service):
                          str(e))
 
     def start_tracking_certificates(self):
+        key_passwd_file = paths.HTTPD_PASSWD_FILE_FMT.format(host=api.env.host)
         cert = x509.load_certificate_from_file(paths.HTTPD_CERT_FILE)
         if certs.is_ipa_issued_cert(api, cert):
             request_id = certmonger.start_tracking(
                 certpath=(paths.HTTPD_CERT_FILE, paths.HTTPD_KEY_FILE),
-                post_command='restart_httpd', storage='FILE'
+                post_command='restart_httpd', storage='FILE',
+                profile=dogtag.DEFAULT_PROFILE,
+                pinfile=key_passwd_file,
             )
             subject = str(DN(cert.subject))
             certmonger.add_principal(request_id, self.principal)
