@@ -5,9 +5,11 @@
 from __future__ import print_function, absolute_import
 
 import errno
+import itertools
 import logging
 import re
 import os
+import glob
 import shutil
 import pwd
 import fileinput
@@ -17,7 +19,9 @@ import tempfile
 from contextlib import contextmanager
 from augeas import Augeas
 import dns.exception
+
 from ipalib import api, x509
+from ipalib.constants import RENEWAL_CA_NAME, RA_AGENT_PROFILE
 from ipalib.install import certmonger, sysrestore
 import SSSDConfig
 import ipalib.util
@@ -27,6 +31,7 @@ from ipaclient.install.client import sssd_enable_ifp
 from ipaplatform import services
 from ipaplatform.tasks import tasks
 from ipapython import ipautil, version
+from ipapython import ipaldap
 from ipapython import dnsutil, directivesetter
 from ipapython.dn import DN
 from ipaplatform.constants import constants
@@ -264,7 +269,7 @@ def cleanup_kdc(fstore):
     logger.info('[Checking for deprecated KDC configuration files]')
     for file in ['kpasswd.keytab', 'ldappwd']:
         filename = os.path.join(paths.VAR_KERBEROS_KRB5KDC_DIR, file)
-        installutils.remove_file(filename)
+        ipautil.remove_file(filename)
         if fstore.has_file(filename):
             fstore.untrack_file(filename)
             logger.debug('Uninstalling %s', filename)
@@ -304,6 +309,15 @@ def cleanup_dogtag():
         instance.clean_pkispawn_files()
 
 
+def cleanup_kdcinfo():
+    """ Remove stale kdcinfo.*|kpasswdinfo.* files generated by SSSD """
+
+    for pattern in ('kdcinfo.*', 'kpasswdinfo.*'):
+        for fname in glob.glob(os.path.join(paths.SSSD_PUBCONF_DIR, pattern)):
+            logger.debug('Removing stale info file %s', fname)
+            os.unlink(fname)
+
+
 def upgrade_adtrust_config():
     """
     Upgrade 'dedicated keytab file' in smb.conf to omit FILE: prefix
@@ -323,6 +337,25 @@ def upgrade_adtrust_config():
     except ipautil.CalledProcessError as e:
         logger.warning("Error updating Samba registry: %s", e)
 
+    logger.info("[Update 'max smbd processes' in Samba configuration "
+                "to prevent unlimited SMBLoris attack amplification]")
+
+    args = [paths.NET, "conf", "getparm", "global", "max smbd processes"]
+
+    try:
+        ipautil.run(args)
+    except ipautil.CalledProcessError as e:
+        if e.returncode == 255:
+            # 'max smbd processes' does not exist
+            args = [paths.NET, "conf", "setparm", "global",
+                    "max smbd processes", "1000"]
+            try:
+                ipautil.run(args)
+            except ipautil.CalledProcessError as e:
+                logger.warning("Error updating Samba registry: %s", e)
+        else:
+            logger.warning("Error updating Samba registry: %s", e)
+
 
 def ca_configure_profiles_acl(ca):
     logger.info('[Authorizing RA Agent to modify profiles]')
@@ -933,64 +966,42 @@ def named_add_crypto_policy():
     return True
 
 
-def certificate_renewal_update(ca, ds, http):
+def certificate_renewal_update(ca, kra, ds, http):
     """
     Update certmonger certificate renewal configuration.
     """
 
     template = paths.CERTMONGER_COMMAND_TEMPLATE
-    serverid = installutils.realm_to_serverid(api.env.realm)
+    serverid = ipaldap.realm_to_serverid(api.env.realm)
 
-    requests = [
-        {
+    requests = []
+
+    dogtag_reqs = ca.tracking_reqs.items()
+    if kra.is_installed():
+        dogtag_reqs = itertools.chain(dogtag_reqs, kra.tracking_reqs.items())
+
+    for nick, profile in dogtag_reqs:
+        req = {
             'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
-            'cert-nickname': 'auditSigningCert cert-pki-ca',
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
+            'cert-nickname': nick,
+            'ca-name': RENEWAL_CA_NAME,
             'cert-presave-command': template % 'stop_pkicad',
             'cert-postsave-command':
-                (template % 'renew_ca_cert "auditSigningCert cert-pki-ca"'),
-        },
-        {
-            'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
-            'cert-nickname': 'ocspSigningCert cert-pki-ca',
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
-            'cert-presave-command': template % 'stop_pkicad',
-            'cert-postsave-command':
-                (template % 'renew_ca_cert "ocspSigningCert cert-pki-ca"'),
-        },
-        {
-            'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
-            'cert-nickname': 'subsystemCert cert-pki-ca',
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
-            'cert-presave-command': template % 'stop_pkicad',
-            'cert-postsave-command':
-                (template % 'renew_ca_cert "subsystemCert cert-pki-ca"'),
-        },
-        {
-            'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
-            'cert-nickname': 'caSigningCert cert-pki-ca',
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
-            'cert-presave-command': template % 'stop_pkicad',
-            'cert-postsave-command':
-                (template % 'renew_ca_cert "caSigningCert cert-pki-ca"'),
-            'template-profile': None,
-        },
-        {
-            'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
-            'cert-nickname': 'Server-Cert cert-pki-ca',
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
-            'cert-presave-command': template % 'stop_pkicad',
-            'cert-postsave-command':
-                (template % 'renew_ca_cert "Server-Cert cert-pki-ca"'),
-        },
+                (template % 'renew_ca_cert "{}"'.format(nick)),
+            'template-profile': profile,
+        }
+        requests.append(req)
+
+    requests.append(
         {
             'cert-file': paths.RA_AGENT_PEM,
             'key-file': paths.RA_AGENT_KEY,
-            'ca-name': 'dogtag-ipa-ca-renew-agent',
+            'ca-name': RENEWAL_CA_NAME,
+            'template-profile': RA_AGENT_PROFILE,
             'cert-presave-command': template % 'renew_ra_cert_pre',
             'cert-postsave-command': template % 'renew_ra_cert',
         },
-    ]
+    )
 
     logger.info("[Update certmonger certificate renewal configuration]")
     if not ca.is_configured():
@@ -1003,7 +1014,7 @@ def certificate_renewal_update(ca, ds, http):
         requests.append(
             {
                 'cert-file': paths.HTTPD_CERT_FILE,
-                'key-storage': paths.HTTPD_KEY_FILE,
+                'key-file': paths.HTTPD_KEY_FILE,
                 'ca-name': 'IPA',
                 'cert-postsave-command': template % 'restart_httpd',
             }
@@ -1031,7 +1042,7 @@ def certificate_renewal_update(ca, ds, http):
                 {
                     'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
                     'cert-nickname': nickname,
-                    'ca-name': 'dogtag-ipa-ca-renew-agent',
+                    'ca-name': RENEWAL_CA_NAME,
                     'cert-presave-command': template % 'stop_pkicad',
                     'cert-postsave-command':
                         (template % ('renew_ca_cert "%s"' % nickname)),
@@ -1040,18 +1051,33 @@ def certificate_renewal_update(ca, ds, http):
             )
 
     # State not set, lets see if we are already configured
+    missing_or_misconfigured_requests = []
     for request in requests:
         request_id = certmonger.get_request_id(request)
         if request_id is None:
-            break
-    else:
+            missing_or_misconfigured_requests.append(request)
+
+    if len(missing_or_misconfigured_requests) == 0:
         logger.info("Certmonger certificate renewal configuration already "
                     "up-to-date")
         return False
 
+    # Print info about missing requests
+    logger.info("Missing or incorrect tracking request for certificates:")
+    for request in missing_or_misconfigured_requests:
+        cert = None
+        if 'cert-file' in request:
+            cert = request['cert-file']
+        elif 'cert-database' in request and 'cert-nickname' in request:
+            cert = '{cert-database}:{cert-nickname}'.format(**request)
+        if cert is not None:
+            logger.info("  %s", cert)
+
     # Ok, now we need to stop tracking, then we can start tracking them
     # again with new configuration:
     ca.stop_tracking_certificates()
+    if kra.is_installed():
+        kra.stop_tracking_certificates()
     ds.stop_tracking_certificates(serverid)
     http.stop_tracking_certificates()
 
@@ -1059,13 +1085,14 @@ def certificate_renewal_update(ca, ds, http):
     if os.path.exists(filename):
         with installutils.stopped_service('certmonger'):
             logger.info("Removing %s", filename)
-            installutils.remove_file(filename)
+            ipautil.remove_file(filename)
 
     ca.configure_certmonger_renewal()
     ca.configure_renewal()
     ca.configure_agent_renewal()
-    ca.track_servercert()
     ca.add_lightweight_ca_tracking_requests()
+    if kra.is_installed():
+        kra.configure_renewal()
     ds.start_tracking_certificates(serverid)
     http.start_tracking_certificates()
 
@@ -1249,8 +1276,8 @@ def uninstall_dogtag_9(ds, http):
         logger.debug('Dogtag is version 10 or above')
         return
 
-    dn = DN(('cn', 'CA'), ('cn', api.env.host), ('cn', 'masters'),
-            ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
+    dn = DN(('cn', 'CA'), ('cn', api.env.host), api.env.container_masters,
+            api.env.basedn)
     try:
         api.Backend.ldap2.delete_entry(dn)
     except ipalib.errors.PublicError as e:
@@ -1357,7 +1384,7 @@ def fix_schema_file_syntax():
         logger.info('Syntax already fixed')
         return
 
-    serverid = installutils.realm_to_serverid(api.env.realm)
+    serverid = ipaldap.realm_to_serverid(api.env.realm)
     ds_dir = dsinstance.config_dirname(serverid)
 
     # 1. 60ipadns.ldif: Add parenthesis to idnsRecord
@@ -1399,15 +1426,6 @@ def fix_schema_file_syntax():
     sysupgrade.set_upgrade_state('ds', 'fix_schema_syntax', True)
 
 
-def set_sssd_domain_option(option, value):
-    sssdconfig = SSSDConfig.SSSDConfig()
-    sssdconfig.import_config()
-    domain = sssdconfig.get_domain(str(api.env.domain))
-    domain.set_option(option, value)
-    sssdconfig.save_domain(domain)
-    sssdconfig.write(paths.SSSD_CONF)
-
-
 def sssd_update():
     sssdconfig = SSSDConfig.SSSDConfig()
     sssdconfig.import_config()
@@ -1416,8 +1434,19 @@ def sssd_update():
     domain.set_option('ipa_server_mode', 'True')
     domain.set_option('ipa_server', api.env.host)
     sssdconfig.save_domain(domain)
+    # check if service has ok_to_auth_as_delegate
+    service = 'HTTP/{}'.format(api.env.host)
+    result = api.Command.service_show(service, all=True)
+    flag = result['result'].get('ipakrboktoauthasdelegate', False)
+    if flag:
+        logger.debug(
+            "%s has ok_to_auth_as_delegate, allow Apache to access IFP",
+            services
+        )
     # enable and configure IFP plugin
-    sssd_enable_ifp(sssdconfig)
+    sssd_enable_ifp(sssdconfig, allow_httpd=flag)
+    # clean stale files generated by sssd
+    cleanup_kdcinfo()
     # write config and restart service
     sssdconfig.write(paths.SSSD_CONF)
     sssd = services.service('sssd', api)
@@ -1432,7 +1461,7 @@ def remove_ds_ra_cert(subject_base):
         return
 
     dbdir = dsinstance.config_dirname(
-        installutils.realm_to_serverid(api.env.realm))
+        ipaldap.realm_to_serverid(api.env.realm))
     dsdb = certs.CertDB(api.env.realm, nssdir=dbdir, subject_base=subject_base)
 
     nickname = 'CN=IPA RA,%s' % subject_base
@@ -1658,20 +1687,20 @@ def ntpd_cleanup(fqdn, fstore):
             instance.disable()
             instance.stop()
         except Exception as e:
-            logger.info("Service ntpd was not disabled or stopped")
+            logger.debug("Service ntpd was not disabled or stopped")
 
     for ntpd_file in [paths.NTP_CONF, paths.NTP_STEP_TICKERS,
                       paths.SYSCONFIG_NTPD]:
         try:
             fstore.restore_file(ntpd_file)
         except ValueError as e:
-            logger.warning(e)
+            logger.debug(e)
 
     try:
         api.Backend.ldap2.delete_entry(DN(('cn', 'NTP'), ('cn', fqdn),
                                        api.env.container_masters))
     except ipalib.errors.NotFound:
-        logger.warning("Warning: NTP service entry was not found in LDAP.")
+        logger.debug("NTP service entry was not found in LDAP.")
 
     ntp_role_instance = servroles.ServiceBasedRole(
          u"ntp_server_server",
@@ -1723,6 +1752,41 @@ def migrate_to_authselect():
     sysupgrade.set_upgrade_state('authcfg', 'migrated_to_authselect', True)
 
 
+def add_systemd_user_hbac():
+    logger.info('[Create systemd-user hbac service and rule]')
+    rule = u'allow_systemd-user'
+    service = u'systemd-user'
+    try:
+        api.Command.hbacsvc_add(
+            service,
+            description=u'pam_systemd and systemd user@.service'
+        )
+    except ipalib.errors.DuplicateEntry:
+        logger.info('hbac service %s already exists', service)
+        # Don't create hbac rule when hbacsvc already exists, so the rule
+        # does not get re-created after it has been deleted by an admin.
+        return
+    else:
+        logger.info('Created hbacsvc %s', service)
+
+    try:
+        api.Command.hbacrule_add(
+            rule,
+            description=(u'Allow pam_systemd to run user@.service to create '
+                         'a system user session'),
+            usercategory=u'all',
+            hostcategory=u'all',
+        )
+    except ipalib.errors.DuplicateEntry:
+        logger.info('hbac rule %s already exists', rule)
+    else:
+        api.Command.hbacrule_add_service(
+            rule,
+            hbacsvc=(service,)
+        )
+        logger.info('Created hbac rule %s with hbacsvc=%s', rule, service)
+
+
 def fix_permissions():
     """Fix permission of public accessible files and directories
 
@@ -1763,17 +1827,19 @@ def upgrade_configuration():
     fqdn = api.env.host
 
     # Ok, we are an IPA server, do the additional tests
-    ds_serverid = installutils.realm_to_serverid(api.env.realm)
-    ds = dsinstance.DsInstance()
+    ds = dsinstance.DsInstance(realm_name=api.env.realm)
 
     # start DS, CA will not start without running DS, and cause error
     ds_running = ds.is_running()
     if not ds_running:
-        ds.start(ds_serverid)
+        ds.start(ds.serverid)
 
     if not sysupgrade.get_upgrade_state('ntpd', 'ntpd_cleaned'):
         ntpd_cleanup(fqdn, fstore)
 
+    if tasks.configure_pkcs11_modules(fstore):
+        print("Disabled p11-kit-proxy")
+
     check_certs()
     fix_permissions()
 
@@ -1789,6 +1855,8 @@ def upgrade_configuration():
         WSGI_PROCESSES=constants.WSGI_PROCESSES,
         GSSAPI_SESSION_KEY=paths.GSSAPI_SESSION_KEY,
         FONTS_DIR=paths.FONTS_DIR,
+        FONTS_OPENSANS_DIR=paths.FONTS_OPENSANS_DIR,
+        FONTS_FONTAWESOME_DIR=paths.FONTS_FONTAWESOME_DIR,
         IPA_CCACHES=paths.IPA_CCACHES,
         IPA_CUSTODIA_SOCKET=paths.IPA_CUSTODIA_SOCKET,
         KDCPROXY_CONFIG=paths.KDCPROXY_CONFIG,
@@ -1823,7 +1891,7 @@ def upgrade_configuration():
                 paths.CA_CS_CFG_PATH, 'ca.crl.MasterCRL.enableCRLUpdates', '=')
             sub_dict['CLONE']='#' if crl.lower() == 'true' else ''
 
-        ds_dirname = dsinstance.config_dirname(ds_serverid)
+        ds_dirname = dsinstance.config_dirname(ds.serverid)
 
         upgrade_file(sub_dict, paths.HTTPD_IPA_CONF,
                      os.path.join(paths.USR_SHARE_IPA_DIR,
@@ -1899,16 +1967,16 @@ def upgrade_configuration():
 
     http.enable_and_start_oddjobd()
 
-    ds.configure_dirsrv_ccache()
+    ds.configure_systemd_ipa_env()
 
     update_replica_config(ipautil.realm_to_suffix(api.env.realm))
     if ca.is_configured():
         update_replica_config(DN(('o', 'ipaca')))
 
-    ds.stop(ds_serverid)
+    ds.stop(ds.serverid)
     fix_schema_file_syntax()
     remove_ds_ra_cert(subject_base)
-    ds.start(ds_serverid)
+    ds.start(ds.serverid)
 
     ds.fqdn = fqdn
     ds.realm = api.env.realm
@@ -2006,7 +2074,7 @@ def upgrade_configuration():
         ca_restart,
         ca_upgrade_schema(ca),
         upgrade_ca_audit_cert_validity(ca),
-        certificate_renewal_update(ca, ds, http),
+        certificate_renewal_update(ca, kra, ds, http),
         ca_enable_pkix(ca),
         ca_configure_profiles_acl(ca),
         ca_configure_lightweight_ca_acls(ca),
@@ -2032,11 +2100,13 @@ def upgrade_configuration():
     add_default_caacl(ca)
 
     if ca.is_configured():
+        ca.reindex_task()
         cainstance.repair_profile_caIPAserviceCert()
         ca.setup_lightweight_ca_key_retrieval()
         cainstance.ensure_ipa_authority_entry()
 
     migrate_to_authselect()
+    add_systemd_user_hbac()
 
     sssd_update()
 
@@ -2049,7 +2119,7 @@ def upgrade_configuration():
                         SUFFIX=krb.suffix,
                         DOMAIN=api.env.domain,
                         HOST=api.env.host,
-                        SERVER_ID=installutils.realm_to_serverid(krb.realm),
+                        SERVER_ID=ipaldap.realm_to_serverid(krb.realm),
                         REALM=krb.realm,
                         KRB5KDC_KADM5_ACL=paths.KRB5KDC_KADM5_ACL,
                         DICT_WORDS=paths.DICT_WORDS,
@@ -2065,7 +2135,7 @@ def upgrade_configuration():
     enable_certauth(krb)
 
     if not ds_running:
-        ds.stop(ds_serverid)
+        ds.stop(ds.serverid)
 
     if ca.is_configured():
         if ca_running and not ca.is_running():
