@@ -389,9 +389,9 @@ class i18n_messages(Command):
             "krb_auth_msg": _(
                 "<i class=\"fa fa-info-circle\"></i> To log in with "
                 "<strong>Kerberos</strong>, please make sure you have valid "
-                "tickets (obtainable via kinit) and <a href='http://${host}/"
-                "ipa/config/ssbrowser.html'>configured</a> the browser "
-                "correctly, then click 'Log in'."),
+                "tickets (obtainable via kinit) and <a href='${protocol}//"
+                "${host}/ipa/config/ssbrowser.html'>configured</a> the browser"
+                " correctly, then click 'Log in'."),
             "loading": _("Loading"),
             "krbprincipal_expired": _(
                 "Kerberos Principal you entered is expired"),
@@ -726,6 +726,7 @@ class i18n_messages(Command):
                 "group": _("Group Options"),
                 "search": _("Search Options"),
                 "selinux": _("SELinux Options"),
+                "server": _("Server Options"),
                 "service": _("Service Options"),
                 "user": _("User Options"),
             },
@@ -1483,6 +1484,23 @@ class i18n_messages(Command):
                 "trusttype": _("Trust type"),
                 "ipantadditionalsuffixes": _("Alternative UPN suffixes"),
             },
+            'smb_attributes': {
+                "title": _(
+                    "User attributes for SMB services"
+                ),
+                "ipantlogonscript_tooltip": _(
+                    "Path to a script executed on a Windows system at logon"
+                ),
+                "ipantprofilepath_tooltip": _(
+                    "Path to a user profile, in UNC format \\\\server\\share\\"
+                ),
+                "ipanthomedirectory_tooltip": _(
+                    "Path to a user home directory, in UNC format"
+                ),
+                "ipanthomedirectorydrive_tooltip": _(
+                    "Drive to mount a home directory"
+                ),
+            },
             "trustconfig": {
                 "options": _("Options"),
             },
