@@ -17,21 +17,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""
-Joining an IPA domain
-"""
+import logging
 
 import six
 
-from ipalib import api
+from ipalib import Registry, api
 from ipalib import Command, Str
 from ipalib import errors
 from ipalib import _
 from ipaserver.install import installutils
 
+__doc__ = _("""
+Joining an IPA domain
+""")
+
 if six.PY3:
     unicode = str
 
+logger = logging.getLogger(__name__)
+
+register = Registry()
+
 
 def validate_host(ugettext, cn):
     """
@@ -43,8 +49,11 @@ def validate_host(ugettext, cn):
     return None
 
 
+@register()
 class join(Command):
-    """Join an IPA domain"""
+    __doc__ = _('Join an IPA domain')
+
+    NO_CLI = True
 
     takes_args = (
         Str('cn',
@@ -95,16 +104,16 @@ class join(Command):
             dn = attrs_list['dn']
 
             # No error raised so far means that host entry exists
-            self.log.info('Host entry for %s already exists, '
-                          'joining may fail on the client side '
-                          'if not forced', hostname)
+            logger.info('Host entry for %s already exists, '
+                        'joining may fail on the client side '
+                        'if not forced', hostname)
 
             # If no principal name is set yet we need to try to add
             # one.
             if 'krbprincipalname' not in attrs_list:
                 service = "host/%s@%s" % (hostname, api.env.realm)
                 api.Command['host_mod'](hostname, krbprincipalname=service)
-                self.log.info('No principal set, setting to %s', service)
+                logger.info('No principal set, setting to %s', service)
 
             # It exists, can we write the password attributes?
             allowed = ldap.can_write(dn, 'krblastpwdchange')
@@ -127,5 +136,3 @@ class join(Command):
             config['ipacertificatesubjectbase']
 
         return (dn, attrs_list)
-
-api.register(join)
