@@ -37,6 +37,7 @@ from ipalib import _, ngettext
 from ipalib import output
 from .hbacrule import is_all
 from ipapython.dn import DN
+from ipaplatform.constants import constants as platformconstants
 
 __doc__ = _("""
 SELinux User Mapping
@@ -92,37 +93,62 @@ def validate_selinuxuser(ugettext, user):
     """
     An SELinux user has 3 components: user:MLS:MCS. user and MLS are required.
     user traditionally ends with _u but this is not mandatory.
-      The regex is ^[a-zA-Z][a-zA-Z_]*
+      The regex is {name}
 
     The MLS part can only be:
-      Level: s[0-15](-s[0-15])
+      Level: {mls}
+      MaxLevel: {mls_max}
 
-    Then MCS could be c[0-1023].c[0-1023] and/or c[0-1023]-c[0-c0123]
-    Meaning
-    s0 s0-s1 s0-s15:c0.c1023 s0-s1:c0,c2,c15.c26 s0-s0:c0.c1023
+    Then MCS could be {mcs}
+    MaxCat: {mcs_max}
 
     Returns a message on invalid, returns nothing on valid.
-    """
-    regex_name = re.compile(r'^[a-zA-Z][a-zA-Z_]*$')
-    regex_mls = re.compile(r'^s[0-9][1-5]{0,1}(-s[0-9][1-5]{0,1}){0,1}$')
-    regex_mcs = re.compile(r'^c(\d+)([.,-]c(\d+))*?$')
+    """.format(
+        name=platformconstants.SELINUX_USER_REGEX,
+        mls=platformconstants.SELINUX_MLS_REGEX,
+        mls_max=platformconstants.SELINUX_MLS_MAX,
+        mcs=platformconstants.SELINUX_MCS_REGEX,
+        mcs_max=platformconstants.SELINUX_MCS_MAX,
+    )
+    SELINUX_MCS_MAX = platformconstants.SELINUX_MCS_MAX
+    SELINUX_MCS_REGEX = platformconstants.SELINUX_MCS_REGEX
+    SELINUX_MLS_MAX = platformconstants.SELINUX_MLS_MAX
+    SELINUX_MLS_REGEX = platformconstants.SELINUX_MLS_REGEX
+    SELINUX_USER_REGEX = platformconstants.SELINUX_USER_REGEX
+
+    regex_name = re.compile(SELINUX_USER_REGEX)
+    regex_mls = re.compile(SELINUX_MLS_REGEX)
+    regex_mcs = re.compile(SELINUX_MCS_REGEX)
 
     # If we add in ::: we don't have to check to see if some values are
     # empty
-    (name, mls, mcs, ignore) = (user + ':::').split(':', 3)
+    (name, mls, mcs, _ignore) = (user + ':::').split(':', 3)
 
     if not regex_name.match(name):
-        return _('Invalid SELinux user name, only a-Z and _ are allowed')
-    if not mls or not regex_mls.match(mls):
-        return _('Invalid MLS value, must match s[0-15](-s[0-15])')
-    m = regex_mcs.match(mcs)
-    if mcs and (not m or (m.group(3) and (int(m.group(3)) > 1023))):
-        return _('Invalid MCS value, must match c[0-1023].c[0-1023] '
-                 'and/or c[0-1023]-c[0-c0123]')
+        return _('Invalid SELinux user name, must match {}').format(
+            SELINUX_USER_REGEX)
+
+    def _validate_level(level, level_regex, upper_limit):
+        if not level_regex.match(level):
+            return False
+
+        for m in re.finditer(r'\d+', level):
+            if int(m.group()) > upper_limit:
+                return False
+        return True
+
+    if not mls or not _validate_level(mls, regex_mls, SELINUX_MLS_MAX):
+        return _(
+            'Invalid MLS value, must match {mls}, where max level '
+            '{mls_max}').format(mls=SELINUX_MLS_REGEX, mls_max=SELINUX_MLS_MAX)
+
+    if mcs and not _validate_level(mcs, regex_mcs, SELINUX_MCS_MAX):
+        return _(
+            'Invalid MCS value, must match {mcs}, where max category '
+            '{mcs_max}').format(mcs=SELINUX_MCS_REGEX, mcs_max=SELINUX_MCS_MAX)
 
     return None
 
-
 def validate_selinuxuser_inlist(ldap, user):
     """
     Ensure the user is in the list of allowed SELinux users.
@@ -140,8 +166,6 @@ def validate_selinuxuser_inlist(ldap, user):
             reason=_('SELinux user %(user)s not found in '
                      'ordering list (in config)') % dict(user=user))
 
-    return
-
 
 @register()
 class selinuxusermap(LDAPObject):
@@ -355,16 +379,24 @@ class selinuxusermap_mod(LDAPUpdate):
         try:
             _entry_attrs = ldap.get_entry(dn, attrs_list)
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
+            raise self.obj.handle_not_found(*keys)
 
-        is_to_be_deleted = lambda x: (x in _entry_attrs and x in entry_attrs) and \
-                                     entry_attrs[x] == None
+        def is_to_be_deleted(x):
+            return (
+                (x in _entry_attrs and x in entry_attrs)
+                and entry_attrs[x] is None
+            )
 
         # makes sure the local members and hbacrule is not set at the same time
         # memberuser or memberhost could have been set using --setattr
-        is_to_be_set = lambda x: ((x in _entry_attrs and _entry_attrs[x] != None) or \
-                                 (x in entry_attrs and entry_attrs[x] != None)) and \
-                                 not is_to_be_deleted(x)
+        def is_to_be_set(x):
+            return (
+                (
+                    (x in _entry_attrs and _entry_attrs[x] is not None) or
+                    (x in entry_attrs and entry_attrs[x] is not None)
+                )
+                and not is_to_be_deleted(x)
+            )
 
         are_local_members_to_be_set = any(is_to_be_set(attr)
                                           for attr in ('usercategory',
@@ -379,18 +411,26 @@ class selinuxusermap_mod(LDAPUpdate):
         if are_local_members_to_be_set and is_hbacrule_to_be_set:
             raise errors.MutuallyExclusiveError(reason=notboth_err)
 
-        if is_all(entry_attrs, 'usercategory') and 'memberuser' in entry_attrs:
-            raise errors.MutuallyExclusiveError(reason="user category "
-                 "cannot be set to 'all' while there are allowed users")
-        if is_all(entry_attrs, 'hostcategory') and 'memberhost' in entry_attrs:
-            raise errors.MutuallyExclusiveError(reason="host category "
-                 "cannot be set to 'all' while there are allowed hosts")
+        if (is_all(entry_attrs, 'usercategory')
+                and 'memberuser' in entry_attrs):
+            raise errors.MutuallyExclusiveError(
+                reason="user category cannot be set to 'all' while there "
+                       "are allowed users"
+            )
+        if (is_all(entry_attrs, 'hostcategory')
+                and 'memberhost' in entry_attrs):
+            raise errors.MutuallyExclusiveError(
+                reason="host category cannot be set to 'all' while there "
+                       "are allowed hosts"
+            )
 
         if 'ipaselinuxuser' in entry_attrs:
             validate_selinuxuser_inlist(ldap, entry_attrs['ipaselinuxuser'])
 
         if 'seealso' in entry_attrs:
-            entry_attrs['seealso'] = self.obj._normalize_seealso(entry_attrs['seealso'])
+            entry_attrs['seealso'] = self.obj._normalize_seealso(
+                entry_attrs['seealso']
+            )
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -457,7 +497,7 @@ class selinuxusermap_enable(LDAPQuery):
         try:
             entry_attrs = ldap.get_entry(dn, ['ipaenabledflag'])
         except errors.NotFound:
-            self.obj.handle_not_found(cn)
+            raise self.obj.handle_not_found(cn)
 
         entry_attrs['ipaenabledflag'] = ['TRUE']
 
@@ -487,7 +527,7 @@ class selinuxusermap_disable(LDAPQuery):
         try:
             entry_attrs = ldap.get_entry(dn, ['ipaenabledflag'])
         except errors.NotFound:
-            self.obj.handle_not_found(cn)
+            raise self.obj.handle_not_found(cn)
 
         entry_attrs['ipaenabledflag'] = ['FALSE']
 
@@ -516,9 +556,9 @@ class selinuxusermap_add_user(LDAPAddMember):
             entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
             dn = entry_attrs.dn
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
-        if 'usercategory' in entry_attrs and \
-            entry_attrs['usercategory'][0].lower() == 'all':
+            raise self.obj.handle_not_found(*keys)
+        if ('usercategory' in entry_attrs and
+                entry_attrs['usercategory'][0].lower() == 'all'):
             raise errors.MutuallyExclusiveError(
                 reason=_("users cannot be added when user category='all'"))
         if 'seealso' in entry_attrs:
@@ -549,9 +589,9 @@ class selinuxusermap_add_host(LDAPAddMember):
             entry_attrs = ldap.get_entry(dn, self.obj.default_attributes)
             dn = entry_attrs.dn
         except errors.NotFound:
-            self.obj.handle_not_found(*keys)
-        if 'hostcategory' in entry_attrs and \
-            entry_attrs['hostcategory'][0].lower() == 'all':
+            raise self.obj.handle_not_found(*keys)
+        if ('hostcategory' in entry_attrs and
+                entry_attrs['hostcategory'][0].lower() == 'all'):
             raise errors.MutuallyExclusiveError(
                 reason=_("hosts cannot be added when host category='all'"))
         if 'seealso' in entry_attrs:
@@ -566,4 +606,3 @@ class selinuxusermap_remove_host(LDAPRemoveMember):
 
     member_attributes = ['memberhost']
     member_count_out = ('%i object removed.', '%i objects removed.')
-
