@@ -44,14 +44,15 @@ from .baseldap import (
     LDAPAddAttribute,
     LDAPRemoveAttribute,
     LDAPAddAttributeViaOption,
-    LDAPRemoveAttributeViaOption)
+    LDAPRemoveAttributeViaOption,
+    DNA_MAGIC)
 from ipalib import x509
 from ipalib import _, ngettext
 from ipalib import util
 from ipalib import output
 from ipapython import kerberos
 from ipapython.dn import DN
-
+from ipapython.dnsutil import DNSName
 
 if six.PY3:
     unicode = str
@@ -276,9 +277,13 @@ def set_certificate_attrs(entry_attrs):
 
 def check_required_principal(ldap, principal):
     """
-    Raise an error if the host of this prinicipal is an IPA master and one
+    Raise an error if the host of this principal is an IPA master and one
     of the principals required for proper execution.
     """
+    if not principal.is_service:
+        # bypass check if principal is not a service principal,
+        # see https://pagure.io/freeipa/issue/7793
+        return
     try:
         host_is_master(ldap, principal.hostname)
     except errors.ValidationError:
@@ -443,6 +448,16 @@ class service(LDAPObject):
             ],
             'default_privileges': {'Service Administrators'},
         },
+        'System: Read POSIX details of SMB services': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'objectclass', 'cn', 'uid', 'gecos', 'gidnumber',
+                'homedirectory', 'loginshell', 'uidnumber',
+                'ipantsecurityidentifier',
+            },
+        }
     }
 
     label = _('Services')
@@ -546,7 +561,7 @@ class service(LDAPObject):
 
     def get_dn(self, *keys, **kwargs):
         key = keys[0]
-        if isinstance(key, six.text_type):
+        if isinstance(key, str):
             key = kerberos.Principal(key)
 
         key = unicode(normalize_principal(key))
@@ -660,6 +675,133 @@ class service_add(LDAPCreate):
         return dn
 
 
+@register()
+class service_add_smb(LDAPCreate):
+    __doc__ = _('Add a new SMB service.')
+
+    msg_summary = _('Added service "%(value)s"')
+    member_attributes = ['managedby']
+    has_output_params = LDAPCreate.has_output_params + output_params
+    smb_takes_args = (
+        Str('fqdn', util.hostname_validator,
+            cli_name='hostname',
+            label=_('Host name'),
+            primary_key=True,
+            normalizer=util.normalize_hostname,
+            flags={'virtual_attribute', 'no_display', 'no_update',
+                   'no_search'},
+            ),
+        Str('ipantflatname?',
+            cli_name='netbiosname',
+            label=_('SMB service NetBIOS name'),
+            flags={'virtual_attribute', 'no_display', 'no_update',
+                   'no_search'},
+            ),
+    )
+
+    takes_options = LDAPCreate.takes_options
+
+    def get_args(self):
+        """
+        Rewrite arguments to service-add-smb command to make sure we accept
+        hostname instead of a principal as we'll be constructing the principal
+        ourselves
+        """
+        for arg in self.smb_takes_args:
+            yield arg
+        for arg in super(service_add_smb, self).get_args():
+            if arg not in self.smb_takes_args and not arg.primary_key:
+                yield arg
+
+    def get_options(self):
+        """
+        Rewrite options to service-add-smb command to filter out cannonical
+        principal which is autoconstructed. Also filter out options which
+        make no sense for SMB service.
+        """
+        excluded = ('ipakrbauthzdata', 'krbprincipalauthind',
+                    'ipakrbrequirespreauth')
+        for arg in self.takes_options:
+            yield arg
+        for arg in super(service_add_smb, self).get_options():
+            check = all([arg not in self.takes_options,
+                         not arg.primary_key,
+                         arg.name not in excluded])
+            if check:
+                yield arg
+
+    def pre_callback(self, ldap, dn, entry_attrs, attrs_list,
+                     *keys, **options):
+        assert isinstance(dn, DN)
+        hostname = keys[0]
+        if len(keys) == 2:
+            netbiosname = keys[1]
+        else:
+            # By default take leftmost label from the host name
+            netbiosname = DNSName.from_text(hostname)[0].decode().upper()
+
+        # SMB service requires existence of the host object
+        # because DCE RPC calls authenticated with GSSAPI are using
+        # host/.. principal by default for validation
+        try:
+            hostresult = self.api.Command['host_show'](hostname)['result']
+        except errors.NotFound:
+            raise errors.NotFound(reason=_(
+                "The host '%s' does not exist to add a service to.") %
+                hostname)
+
+        # We cannot afford the host not being resolvable even for
+        # clustered environments with CTDB because the target name
+        # has to exist even in that case
+        util.verify_host_resolvable(hostname)
+
+        smbaccount = '{name}$'.format(name=netbiosname)
+        smbprincipal = 'cifs/{hostname}'.format(hostname=hostname)
+
+        entry_attrs['krbprincipalname'] = [
+            str(kerberos.Principal(smbprincipal, realm=self.api.env.realm)),
+            str(kerberos.Principal(smbaccount, realm=self.api.env.realm))]
+
+        entry_attrs['krbcanonicalname'] = entry_attrs['krbprincipalname'][0]
+
+        # Rewrite DN using proper rdn and new canonical name because when
+        # LDAPCreate.execute() was called, it set DN to krbcanonicalname=$value
+        dn = DN(('krbprincipalname', entry_attrs['krbcanonicalname']),
+                DN(self.obj.container_dn, api.env.basedn))
+
+        # Enforce ipaKrbPrincipalAlias to aid case-insensitive searches as
+        # krbPrincipalName/krbCanonicalName are case-sensitive in Kerberos
+        # schema
+        entry_attrs['ipakrbprincipalalias'] = entry_attrs['krbcanonicalname']
+
+        for o in ('ipakrbprincipal', 'ipaidobject', 'krbprincipalaux',
+                  'posixaccount'):
+            if o not in entry_attrs['objectclass']:
+                entry_attrs['objectclass'].append(o)
+
+        entry_attrs['uid'] = ['/'.join(
+            kerberos.Principal(smbprincipal).components)]
+        entry_attrs['uid'].append(smbaccount)
+        entry_attrs['cn'] = netbiosname
+        entry_attrs['homeDirectory'] = '/dev/null'
+        entry_attrs['uidNumber'] = DNA_MAGIC
+        entry_attrs['gidNumber'] = DNA_MAGIC
+
+        self.obj.validate_ipakrbauthzdata(entry_attrs)
+
+        if 'managedby' not in entry_attrs:
+            entry_attrs['managedby'] = hostresult['dn']
+
+        update_krbticketflags(ldap, entry_attrs, attrs_list, options, False)
+
+        return dn
+
+    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
+        set_kerberos_attrs(entry_attrs, options)
+        rename_ipaallowedtoperform_from_ldap(entry_attrs, options)
+        self.obj.populate_krbcanonicalname(entry_attrs, options)
+        return dn
+
 
 @register()
 class service_del(LDAPDelete):
@@ -709,7 +851,8 @@ class service_mod(LDAPUpdate):
             removed_certs = set(old_certs) - set(certs)
             for cert in removed_certs:
                 rm_certs = api.Command.cert_find(
-                    certificate=cert.public_bytes(x509.Encoding.DER))['result']
+                    certificate=cert.public_bytes(x509.Encoding.DER),
+                    service=keys)['result']
                 revoke_certs(rm_certs)
 
         if certs:
@@ -746,7 +889,6 @@ class service_find(LDAPSearch):
         assert isinstance(base_dn, DN)
         # lisp style!
         custom_filter = '(&(objectclass=ipaService)' \
-                          '(!(objectClass=posixAccount))' \
                           '(!(|(krbprincipalname=kadmin/*)' \
                               '(krbprincipalname=K/M@*)' \
                               '(krbprincipalname=krbtgt/*))' \
@@ -989,7 +1131,9 @@ class service_remove_cert(LDAPRemoveAttributeViaOption):
         assert isinstance(dn, DN)
 
         for cert in options.get('usercertificate', []):
-            revoke_certs(api.Command.cert_find(certificate=cert)['result'])
+            revoke_certs(api.Command.cert_find(
+                certificate=cert,
+                service=keys)['result'])
 
         return dn
 
