@@ -5,34 +5,33 @@ from __future__ import print_function
 
 import os
 import pprint
+import shutil
 import sys
+import tempfile
 
 import pytest
 
 from ipalib import api
 from ipalib.cli import cli_plugins
+import ipatests.util
 
 try:
     import ipaplatform  # pylint: disable=unused-import
 except ImportError:
     ipaplatform = None
-try:
-    import ipaserver
-    from ipaserver.install import installutils
-except ImportError:
-    ipaserver = None
+
 
 HERE = os.path.dirname(os.path.abspath(__file__))
 
 pytest_plugins = [
     'ipatests.pytest_ipa.additional_config',
+    'ipatests.pytest_ipa.slicing',
     'ipatests.pytest_ipa.beakerlib',
     'ipatests.pytest_ipa.declarative',
     'ipatests.pytest_ipa.nose_compat',
+    'ipatests.pytest_ipa.integration',
+    'pytester',
 ]
-# The integration plugin is not available in client-only builds.
-if ipaserver is not None:
-    pytest_plugins.append('ipatests.pytest_ipa.integration')
 
 
 MARKERS = [
@@ -88,6 +87,11 @@ def pytest_configure(config):
     # always run doc tests
     config.option.doctestmodules = True
 
+    # apply global options
+    ipatests.util.SKIP_IPAAPI = config.option.skip_ipaapi
+    ipatests.util.IPACLIENT_UNITTESTS = config.option.ipaclient_unittests
+    ipatests.util.PRETTY_PRINT = config.option.pretty_print
+
 
 def pytest_addoption(parser):
     group = parser.getgroup("IPA integration tests")
@@ -117,7 +121,7 @@ def pytest_cmdline_main(config):
 
     # XXX workaround until https://fedorahosted.org/freeipa/ticket/6408 has
     # been resolved.
-    if ipaserver is not None and installutils.is_ipa_configured():
+    if os.path.isfile(api.env.conf_default):
         api.finalize()
 
     if config.option.verbose:
@@ -131,11 +135,28 @@ def pytest_cmdline_main(config):
 
 def pytest_runtest_setup(item):
     if isinstance(item, pytest.Function):
-        if item.get_marker('skip_ipaclient_unittest'):
+        # pytest 3.6 has deprecated get_marker in 3.6. The method was
+        # removed in 4.x and replaced with get_closest_marker.
+        if hasattr(item, 'get_closest_marker'):
+            get_marker = item.get_closest_marker  # pylint: disable=no-member
+        else:
+            get_marker = item.get_marker  # pylint: disable=no-member
+        if get_marker('skip_ipaclient_unittest'):
             # pylint: disable=no-member
-            if pytest.config.option.ipaclient_unittests:
+            if item.config.option.ipaclient_unittests:
                 pytest.skip("Skip in ipaclient unittest mode")
-        if item.get_marker('needs_ipaapi'):
+        if get_marker('needs_ipaapi'):
             # pylint: disable=no-member
-            if pytest.config.option.skip_ipaapi:
+            if item.config.option.skip_ipaapi:
                 pytest.skip("Skip tests that needs an IPA API")
+
+
+@pytest.fixture
+def tempdir(request):
+    tempdir = tempfile.mkdtemp()
+
+    def fin():
+        shutil.rmtree(tempdir)
+
+    request.addfinalizer(fin)
+    return tempdir
