@@ -23,6 +23,7 @@ from __future__ import absolute_import
 
 import logging
 import os
+from io import StringIO
 import textwrap
 import re
 import collections
@@ -33,15 +34,15 @@ import time
 import dns
 from ldif import LDIFWriter
 import pytest
-from SSSDConfig import SSSDConfig
-from six import StringIO
+from cryptography import x509
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.backends import default_backend
 
-
+from ipapython import certdb
 from ipapython import ipautil
 from ipaplatform.paths import paths
+from ipaplatform.services import knownservices
 from ipapython.dn import DN
 from ipalib import errors
 from ipalib.util import get_reverse_zone_default, verify_host_resolvable
@@ -53,6 +54,7 @@ from ipalib.constants import (
 from ipatests.create_external_ca import ExternalCA
 from .env_config import env_to_script
 from .host import Host
+from .firewall import Firewall
 
 logger = logging.getLogger(__name__)
 
@@ -302,12 +304,7 @@ def enable_replication_debugging(host, log_level=0):
         replace: nsslapd-errorlog-level
         nsslapd-errorlog-level: {log_level}
         """.format(log_level=log_level))
-    host.run_command(['ldapmodify', '-x',
-                      '-D', str(host.config.dirman_dn),
-                      '-w', host.config.dirman_password,
-                      '-h', host.hostname],
-                     stdin_text=logging_ldif)
-
+    ldapmodify_dm(host, logging_ldif)
 
 def set_default_ttl_for_ipa_dns_zone(host, raiseonerr=True):
     args = [
@@ -330,6 +327,8 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
     setup_server_logs_collecting(host)
     apply_common_fixes(host)
     fix_apache_semaphores(host)
+    fw = Firewall(host)
+    fw_services = ["freeipa-ldap", "freeipa-ldaps"]
 
     args = [
         'ipa-server-install',
@@ -348,16 +347,20 @@ def install_master(host, setup_dns=True, setup_kra=False, setup_adtrust=False,
             '--forwarder', host.config.dns_forwarder,
             '--auto-reverse'
         ])
+        fw_services.append("dns")
     if setup_kra:
         args.append('--setup-kra')
     if setup_adtrust:
         args.append('--setup-adtrust')
+        fw_services.append("freeipa-trust")
     if external_ca:
         args.append('--external-ca')
 
     args.extend(extra_args)
     result = host.run_command(args, raiseonerr=raiseonerr,
                               stdin_text=stdin_text)
+    if result.returncode == 0:
+        fw.enable_services(fw_services)
     if result.returncode == 0 and not external_ca:
         # external CA step 1 doesn't have DS and KDC fully configured, yet
         enable_replication_debugging(host)
@@ -411,30 +414,53 @@ def master_authoritative_for_client_domain(master, client):
     return result.returncode == 0
 
 
-def _config_replica_resolvconf_with_master_data(master, replica):
+def config_host_resolvconf_with_master_data(master, host):
     """
-    Configure replica /etc/resolv.conf to use master as DNS server
+    Configure host /etc/resolv.conf to use master as DNS server
     """
     content = ('search {domain}\nnameserver {master_ip}'
                .format(domain=master.domain.name, master_ip=master.ip))
-    replica.put_file_contents(paths.RESOLV_CONF, content)
+    host.put_file_contents(paths.RESOLV_CONF, content)
 
 
 def install_replica(master, replica, setup_ca=True, setup_dns=False,
                     setup_kra=False, setup_adtrust=False, extra_args=(),
                     domain_level=None, unattended=True, stdin_text=None,
-                    raiseonerr=True):
+                    raiseonerr=True, promote=True):
+    """
+    This task installs client and then promote it to the replica
+    """
+    replica_args = list(extra_args)  # needed for client's ntp options
     if domain_level is None:
         domain_level = domainlevel(master)
     check_domain_level(domain_level)
     apply_common_fixes(replica)
     setup_server_logs_collecting(replica)
     allow_sync_ptr(master)
+    fw = Firewall(replica)
+    fw_services = ["freeipa-ldap", "freeipa-ldaps"]
     # Otherwise ipa-client-install would not create a PTR
     # and replica installation would fail
     args = ['ipa-replica-install',
-            '-p', replica.config.dirman_password,
-            '-w', replica.config.admin_password]
+            '--admin-password', replica.config.admin_password]
+
+    if promote:  # while promoting we use directory manager password
+        args.extend(['--password', replica.config.dirman_password])
+        # install client on a replica machine and then promote it to replica
+        # to configure ntp options we have to pass them to client installation
+        # because promotion does not support NTP options
+        ntp_args = [arg for arg in replica_args if "-ntp" in arg]
+
+        for ntp_arg in ntp_args:
+            replica_args.remove(ntp_arg)
+
+        install_client(master, replica, extra_args=ntp_args)
+    else:
+        # for one step installation of replica we need authorized user
+        # to enroll a replica and master server to contact
+        args.extend(['--principal', replica.config.admin_name,
+                     '--server', master.hostname])
+
     if unattended:
         args.append('-U')
     if setup_ca:
@@ -447,17 +473,19 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
             '--setup-dns',
             '--forwarder', replica.config.dns_forwarder
         ])
+        fw_services.append("dns")
     if setup_adtrust:
         args.append('--setup-adtrust')
+        fw_services.append("freeipa-trust")
     if master_authoritative_for_client_domain(master, replica):
         args.extend(['--ip-address', replica.ip])
 
-    args.extend(extra_args)
+    args.extend(replica_args)  # append extra arguments to installation
 
-    # install client on a replica machine and then promote it to replica
-    install_client(master, replica)
     fix_apache_semaphores(replica)
-    args.extend(['-r', replica.domain.realm])
+    args.extend(['--realm', replica.domain.realm,
+                 '--domain', replica.domain.name])
+    fw.enable_services(fw_services)
 
     result = replica.run_command(args, raiseonerr=raiseonerr,
                                  stdin_text=stdin_text)
@@ -465,11 +493,13 @@ def install_replica(master, replica, setup_ca=True, setup_dns=False,
         enable_replication_debugging(replica)
         setup_sssd_debugging(replica)
         kinit_admin(replica)
+    else:
+        fw.disable_services(fw_services)
     return result
 
 
-def install_client(master, client, extra_args=(),
-                   user=None, password=None):
+def install_client(master, client, extra_args=[], user=None,
+                   password=None, unattended=True, stdin_text=None):
     client.collect_log(paths.IPACLIENT_INSTALL_LOG)
 
     apply_common_fixes(client)
@@ -486,17 +516,27 @@ def install_client(master, client, extra_args=(),
     if password is None:
         password = client.config.admin_password
 
-    client.run_command(['ipa-client-install', '-U',
-                        '--domain', client.domain.name,
-                        '--realm', client.domain.realm,
-                        '-p', user,
-                        '-w', password,
-                        '--server', master.hostname]
-                       + list(extra_args))
+    args = [
+        'ipa-client-install',
+        '--domain', client.domain.name,
+        '--realm', client.domain.realm,
+        '-p', user,
+        '-w', password,
+        '--server', master.hostname
+    ]
+
+    if unattended:
+        args.append('-U')
+
+    args.extend(extra_args)
+
+    result = client.run_command(args, stdin_text=stdin_text)
 
     setup_sssd_debugging(client)
     kinit_admin(client)
 
+    return result
+
 
 def install_adtrust(host):
     """
@@ -513,16 +553,12 @@ def install_adtrust(host):
                       '-a', host.config.admin_password,
                       '--add-sids'])
 
+    Firewall(host).enable_service("freeipa-trust")
+
     # Restart named because it lost connection to dirsrv
     # (Directory server restarts during the ipa-adtrust-install)
-    # we use two services named and named-pkcs11,
-    # if named is masked restart named-pkcs11
-    result = host.run_command(['systemctl', 'is-enabled', 'named'],
-                              raiseonerr=False)
-    if result.stdout_text.startswith("masked"):
-        host.run_command(['systemctl', 'restart', 'named-pkcs11'])
-    else:
-        host.run_command(['systemctl', 'restart', 'named'])
+    host.run_command(['systemctl', 'restart',
+                      knownservices.named.systemd_name])
 
     # Check that named is running and has loaded the information from LDAP
     dig_command = ['dig', 'SRV', '+short', '@localhost',
@@ -533,15 +569,94 @@ def install_adtrust(host):
     run_repeatedly(host, dig_command, test=dig_test)
 
 
+def disable_dnssec_validation(host):
+    backup_file(host, paths.NAMED_CONF)
+    named_conf = host.get_file_contents(paths.NAMED_CONF)
+    named_conf = re.sub(br'dnssec-validation\s*yes;', b'dnssec-validation no;',
+                        named_conf)
+    host.put_file_contents(paths.NAMED_CONF, named_conf)
+    restart_named(host)
+
+
+def restore_dnssec_validation(host):
+    restore_files(host)
+    restart_named(host)
+
+
+def is_subdomain(subdomain, domain):
+    subdomain_unpacked = subdomain.split('.')
+    domain_unpacked = domain.split('.')
+
+    subdomain_unpacked.reverse()
+    domain_unpacked.reverse()
+
+    subdomain = False
+
+    if len(subdomain_unpacked) > len(domain_unpacked):
+        subdomain = True
+
+        for subdomain_segment, domain_segment in zip(subdomain_unpacked,
+                                                     domain_unpacked):
+            subdomain = subdomain and subdomain_segment == domain_segment
+
+    return subdomain
+
 def configure_dns_for_trust(master, ad):
     """
-    This method is intentionally left empty. Originally it served for DNS
-    configuration on IPA master according to the relationship of the IPA's
-    and AD's domains.
+    This configures DNS on IPA master according to the relationship of the
+    IPA's and AD's domains.
     """
 
+    kinit_admin(master)
 
-def establish_trust_with_ad(master, ad_domain, extra_args=()):
+    if is_subdomain(ad.domain.name, master.domain.name):
+        master.run_command(['ipa', 'dnsrecord-add', master.domain.name,
+                            '%s.%s' % (ad.shortname, ad.netbios),
+                            '--a-ip-address', ad.ip])
+
+        master.run_command(['ipa', 'dnsrecord-add', master.domain.name,
+                            ad.netbios,
+                            '--ns-hostname',
+                            '%s.%s' % (ad.shortname, ad.netbios)])
+
+        master.run_command(['ipa', 'dnszone-mod', master.domain.name,
+                            '--allow-transfer', ad.ip])
+    else:
+        disable_dnssec_validation(master)
+        master.run_command(['ipa', 'dnsforwardzone-add', ad.domain.name,
+                            '--forwarder', ad.ip,
+                            '--forward-policy', 'only',
+                            ])
+
+
+def unconfigure_dns_for_trust(master, ad):
+    """
+    This undoes changes made by configure_dns_for_trust
+    """
+    kinit_admin(master)
+    if is_subdomain(ad.domain.name, master.domain.name):
+        master.run_command(['ipa', 'dnsrecord-del', master.domain.name,
+                            '%s.%s' % (ad.shortname, ad.netbios),
+                            '--a-rec', ad.ip])
+        master.run_command(['ipa', 'dnsrecord-del', master.domain.name,
+                            ad.netbios,
+                            '--ns-rec', '%s.%s' % (ad.shortname, ad.netbios)])
+    else:
+        master.run_command(['ipa', 'dnsforwardzone-del', ad.domain.name])
+        restore_dnssec_validation(master)
+
+
+def configure_windows_dns_for_trust(ad, master):
+    ad.run_command(['dnscmd', '/zoneadd', master.domain.name,
+                    '/Forwarder', master.ip])
+
+
+def unconfigure_windows_dns_for_trust(ad, master):
+    ad.run_command(['dnscmd', '/zonedelete', master.domain.name, '/f'])
+
+
+def establish_trust_with_ad(master, ad_domain, extra_args=(),
+                            shared_secret=None):
     """
     Establishes trust with Active Directory. Trust type is detected depending
     on the presence of SfU (Services for Unix) support on the AD.
@@ -551,6 +666,7 @@ def establish_trust_with_ad(master, ad_domain, extra_args=()):
     """
 
     # Force KDC to reload MS-PAC info by trying to get TGT for HTTP
+    extra_args = list(extra_args)
     master.run_command(['kinit', '-kt', paths.HTTP_KEYTAB,
                         'HTTP/%s' % master.hostname])
     master.run_command(['systemctl', 'restart', 'krb5kdc.service'])
@@ -560,12 +676,15 @@ def establish_trust_with_ad(master, ad_domain, extra_args=()):
     master.run_command(['klist'])
     master.run_command(['smbcontrol', 'all', 'debug', '100'])
 
-    run_repeatedly(master,
-                   ['ipa', 'trust-add',
-                    '--type', 'ad', ad_domain,
-                    '--admin', 'Administrator',
-                    '--password'] + list(extra_args),
-                   stdin_text=master.config.ad_admin_password)
+    if shared_secret:
+        extra_args += ['--trust-secret']
+        stdin_text = shared_secret
+    else:
+        extra_args += ['--admin', 'Administrator', '--password']
+        stdin_text = master.config.ad_admin_password
+    run_repeatedly(
+        master, ['ipa', 'trust-add', '--type', 'ad', ad_domain] + extra_args,
+        stdin_text=stdin_text)
     master.run_command(['smbcontrol', 'all', 'debug', '1'])
     clear_sssd_cache(master)
     master.run_command(['systemctl', 'restart', 'krb5kdc.service'])
@@ -658,6 +777,7 @@ def modify_sssd_conf(host, domain, mod_dict, provider='ipa',
     :param provider_subtype: backend subtype (e.g. id or sudo), will be added
         to the domain config if not present
     """
+    from SSSDConfig import SSSDConfig
     fd, temp_config_file = tempfile.mkstemp()
     os.close(fd)
     try:
@@ -721,34 +841,46 @@ def sync_time(host, server):
 
     host.run_command(['systemctl', 'stop', 'chronyd'])
     host.run_command(['chronyd', '-q',
-                      "server {srv} iburst".format(srv=server.hostname),
-                      'pidfile /tmp/chronyd.pid', 'bindcmdaddress /'])
+                      "server {srv} iburst maxdelay 1000".format(
+                          srv=server.hostname),
+                      'pidfile /tmp/chronyd.pid', 'bindcmdaddress /',
+                      'maxdistance 1000', 'maxjitter 1000'])
 
 
-def connect_replica(master, replica, domain_level=None):
+def connect_replica(master, replica, domain_level=None,
+                    database=DOMAIN_SUFFIX_NAME):
     if domain_level is None:
         domain_level = master.config.domain_level
     check_domain_level(domain_level)
     if domain_level == DOMAIN_LEVEL_0:
-        replica.run_command(['ipa-replica-manage', 'connect', master.hostname])
+        if database == DOMAIN_SUFFIX_NAME:
+            cmd = 'ipa-replica-manage'
+        else:
+            cmd = 'ipa-csreplica-manage'
+        replica.run_command([cmd, 'connect', master.hostname])
     else:
         kinit_admin(master)
-        master.run_command(["ipa", "topologysegment-add", DOMAIN_SUFFIX_NAME,
+        master.run_command(["ipa", "topologysegment-add", database,
                             "%s-to-%s" % (master.hostname, replica.hostname),
                             "--leftnode=%s" % master.hostname,
                             "--rightnode=%s" % replica.hostname
                             ])
 
 
-def disconnect_replica(master, replica, domain_level=None):
+def disconnect_replica(master, replica, domain_level=None,
+                       database=DOMAIN_SUFFIX_NAME):
     if domain_level is None:
         domain_level = master.config.domain_level
     check_domain_level(domain_level)
     if domain_level == DOMAIN_LEVEL_0:
-        replica.run_command(['ipa-replica-manage', 'disconnect', master.hostname])
+        if database == DOMAIN_SUFFIX_NAME:
+            cmd = 'ipa-replica-manage'
+        else:
+            cmd = 'ipa-csreplica-manage'
+        replica.run_command([cmd, 'disconnect', master.hostname])
     else:
         kinit_admin(master)
-        master.run_command(["ipa", "topologysegment-del", DOMAIN_SUFFIX_NAME,
+        master.run_command(["ipa", "topologysegment-del", database,
                             "%s-to-%s" % (master.hostname, replica.hostname),
                             "--continue"
                             ])
@@ -777,6 +909,9 @@ def uninstall_master(host, ignore_topology_disconnect=True,
 
     result = host.run_command(uninstall_cmd)
     assert "Traceback" not in result.stdout_text
+    if clean:
+        Firewall(host).disable_services(["freeipa-ldap", "freeipa-ldaps",
+                                         "freeipa-trust", "dns"])
 
     host.run_command(['pkidestroy', '-s', 'CA', '-i', 'pki-tomcat'],
                      raiseonerr=False)
@@ -1107,7 +1242,7 @@ def install_topo(topo, master, replicas, clients, domain_level=None,
             logger.info('Connecting replica %s to %s', parent, child)
             connect_replica(parent, child)
         else:
-            logger.info('Installing replica %s from %s', parent, child)
+            logger.info('Installing replica %s from %s', child, parent)
             install_replica(
                 parent, child,
                 setup_ca=setup_replica_cas,
@@ -1314,13 +1449,16 @@ def install_ca(host, domain_level=None, first_instance=False,
     return result
 
 
-def install_dns(host, raiseonerr=True):
+def install_dns(host, raiseonerr=True, extra_args=()):
     args = [
         "ipa-dns-install",
         "--forwarder", host.config.dns_forwarder,
         "-U",
     ]
-    return host.run_command(args, raiseonerr=raiseonerr)
+    args.extend(extra_args)
+    ret = host.run_command(args, raiseonerr=raiseonerr)
+    Firewall(host).enable_service("dns")
+    return ret
 
 
 def uninstall_replica(master, replica):
@@ -1376,6 +1514,49 @@ def run_certutil(host, args, reqdir, dbtype=None,
                             stdin_text=stdin)
 
 
+def certutil_certs_keys(host, reqdir, pwd_file, token_name=None):
+    """Run certutils and get mappings of cert and key files
+    """
+    base_args = ['-f', pwd_file]
+    if token_name is not None:
+        base_args.extend(['-h', token_name])
+    cert_args = base_args + ['-L']
+    key_args = base_args + ['-K']
+
+    result = run_certutil(host, cert_args, reqdir)
+    certs = {}
+    for line in result.stdout_text.splitlines():
+        mo = certdb.CERT_RE.match(line)
+        if mo:
+            certs[mo.group('nick')] = mo.group('flags')
+
+    result = run_certutil(host, key_args, reqdir)
+    assert 'orphan' not in result.stdout_text
+    keys = {}
+    for line in result.stdout_text.splitlines():
+        mo = certdb.KEY_RE.match(line)
+        if mo:
+            keys[mo.group('nick')] = mo.group('keyid')
+    return certs, keys
+
+
+def certutil_fetch_cert(host, reqdir, pwd_file, nickname, token_name=None):
+    """Run certutil and retrieve a cert as cryptography.x509 object
+    """
+    args = ['-f', pwd_file, '-L', '-a', '-n']
+    if token_name is not None:
+        args.extend([
+            '{}:{}'.format(token_name, nickname),
+            '-h', token_name
+        ])
+    else:
+        args.append(nickname)
+    result = run_certutil(host, args, reqdir)
+    return x509.load_pem_x509_certificate(
+        result.stdout_bytes, default_backend()
+    )
+
+
 def upload_temp_contents(host, contents, encoding='utf-8'):
     """Upload contents to a temporary file
 
@@ -1402,7 +1583,8 @@ def assert_error(result, stderr_text, returncode=None):
 def restart_named(*args):
     time.sleep(20)  # give a time to DNSSEC daemons to provide keys for named
     for host in args:
-        host.run_command(["systemctl", "restart", "named-pkcs11.service"])
+        host.run_command(['systemctl', 'restart',
+                          knownservices.named.systemd_name])
     time.sleep(20)  # give a time to named to be ready (zone loading)
 
 
@@ -1467,7 +1649,7 @@ def ldappasswd_user_change(user, oldpw, newpw, master):
     basedn = master.domain.basedn
 
     userdn = "uid={},{},{}".format(user, container_user, basedn)
-    master_ldap_uri = "ldap://{}".format(master.external_hostname)
+    master_ldap_uri = "ldap://{}".format(master.hostname)
 
     args = [paths.LDAPPASSWD, '-D', userdn, '-w', oldpw, '-a', oldpw,
             '-s', newpw, '-x', '-ZZ', '-H', master_ldap_uri]
@@ -1479,7 +1661,7 @@ def ldappasswd_sysaccount_change(user, oldpw, newpw, master):
     basedn = master.domain.basedn
 
     userdn = "uid={},{},{}".format(user, container_sysaccounts, basedn)
-    master_ldap_uri = "ldap://{}".format(master.external_hostname)
+    master_ldap_uri = "ldap://{}".format(master.hostname)
 
     args = [paths.LDAPPASSWD, '-D', userdn, '-w', oldpw, '-a', oldpw,
             '-s', newpw, '-x', '-ZZ', '-H', master_ldap_uri]
@@ -1512,7 +1694,9 @@ def add_dns_zone(master, zone, skip_overlap_check=False,
         logger.debug('Zone %s already added.', zone)
 
 
-def sign_ca_and_transport(host, csr_name, root_ca_name, ipa_ca_name):
+def sign_ca_and_transport(host, csr_name, root_ca_name, ipa_ca_name,
+                          root_ca_path_length=None, ipa_ca_path_length=1,
+                          key_size=None, root_ca_extensions=()):
     """
     Sign ipa csr and save signed CA together with root CA back to the host.
     Returns root CA and IPA CA paths on the host.
@@ -1523,11 +1707,14 @@ def sign_ca_and_transport(host, csr_name, root_ca_name, ipa_ca_name):
     # Get IPA CSR as bytes
     ipa_csr = host.get_file_contents(csr_name)
 
-    external_ca = ExternalCA()
+    external_ca = ExternalCA(key_size=key_size)
     # Create root CA
-    root_ca = external_ca.create_ca()
+    root_ca = external_ca.create_ca(
+        path_length=root_ca_path_length,
+        extensions=root_ca_extensions,
+    )
     # Sign CSR
-    ipa_ca = external_ca.sign_csr(ipa_csr)
+    ipa_ca = external_ca.sign_csr(ipa_csr, path_length=ipa_ca_path_length)
 
     root_ca_fname = os.path.join(test_dir, root_ca_name)
     ipa_ca_fname = os.path.join(test_dir, ipa_ca_name)
@@ -1536,7 +1723,7 @@ def sign_ca_and_transport(host, csr_name, root_ca_name, ipa_ca_name):
     host.put_file_contents(root_ca_fname, root_ca)
     host.put_file_contents(ipa_ca_fname, ipa_ca)
 
-    return (root_ca_fname, ipa_ca_fname)
+    return root_ca_fname, ipa_ca_fname
 
 
 def generate_ssh_keypair():
@@ -1552,6 +1739,7 @@ def generate_ssh_keypair():
 
     pem = key.private_bytes(
         encoding=serialization.Encoding.PEM,
+        # paramiko does not support PKCS#8 format, yet.
         format=serialization.PrivateFormat.TraditionalOpenSSL,
         encryption_algorithm=serialization.NoEncryption()
     )
@@ -1576,9 +1764,71 @@ def strip_cert_header(pem):
         return pem
 
 
-def user_add(host, login):
-    host.run_command([
+def user_add(host, login, first='test', last='user', extra_args=()):
+    cmd = [
         "ipa", "user-add", login,
-        "--first", "test",
-        "--last", "user"
-    ])
+        "--first", first,
+        "--last", last
+    ]
+    cmd.extend(extra_args)
+    return host.run_command(cmd)
+
+
+def group_add(host, groupname, extra_args=()):
+    cmd = [
+        "ipa", "group-add", groupname,
+    ]
+    cmd.extend(extra_args)
+    return host.run_command(cmd)
+
+
+def ldapmodify_dm(host, ldif_text, **kwargs):
+    """Run ldapmodify as Directory Manager
+
+    :param host: host object
+    :param ldif_text: ldif string
+    :param kwargs: additional keyword arguments to run_command()
+    :return: result object
+    """
+    # no hard-coded hostname, let ldapmodify pick up the host from ldap.conf.
+    args = [
+        'ldapmodify',
+        '-x',
+        '-D', str(host.config.dirman_dn),  # pylint: disable=no-member
+        '-w', host.config.dirman_password
+    ]
+    return host.run_command(args, stdin_text=ldif_text, **kwargs)
+
+
+def ldapsearch_dm(host, base, ldap_args, scope='sub', **kwargs):
+    """Run ldapsearch as Directory Manager
+
+    :param host: host object
+    :param base: Base DN
+    :param ldap_args: additional arguments to ldapsearch (filter, attributes)
+    :param scope: search scope (base, sub, one)
+    :param kwargs: additional keyword arguments to run_command()
+    :return: result object
+    """
+    args = [
+        'ldapsearch',
+        '-x', '-ZZ',
+        '-h', host.hostname,
+        '-p', '389',
+        '-D', str(host.config.dirman_dn),  # pylint: disable=no-member
+        '-w', host.config.dirman_password,
+        '-s', scope,
+        '-b', base,
+        '-o', 'ldif-wrap=no',
+        '-LLL',
+    ]
+    args.extend(ldap_args)
+    return host.run_command(args, **kwargs)
+
+
+def create_temp_file(host, directory=None):
+    """Creates temproray file using mktemp."""
+    cmd = ['mktemp']
+    if directory is not None:
+        cmd += ['-p', directory]
+    return host.run_command(cmd).stdout_text
