@@ -23,11 +23,11 @@ import logging
 import os
 import re
 import contextlib
-from tempfile import NamedTemporaryFile
 import pytest
 
 from ipaplatform.constants import constants
 from ipaplatform.paths import paths
+from ipaplatform.tasks import tasks as platformtasks
 from ipapython.dn import DN
 from ipapython import ipautil
 from ipatests.test_integration.base import IntegrationTest
@@ -228,6 +228,9 @@ class TestBackupAndRestore(IntegrationTest):
             finally:
                 self.master.run_command(['userdel', 'ipatest_user1'])
 
+    @pytest.mark.skipif(
+        not platformtasks.is_selinux_enabled(),
+        reason="Test needs SELinux enabled")
     def test_full_backup_and_restore_with_selinux_booleans_off(self):
         """regression test for https://fedorahosted.org/freeipa/ticket/4157"""
         with restore_checker(self.master):
@@ -489,6 +492,15 @@ class TestBackupAndRestoreWithReplica(IntegrationTest):
             domain_level = cls.master.config.domain_level
         else:
             domain_level = cls.domain_level
+        # Configure /etc/resolv.conf on each replica to use the master as DNS
+        # Otherwise ipa-replica-manage re-initialize is unable to
+        # resolve the master name
+        tasks.config_host_resolvconf_with_master_data(
+            cls.master, cls.replica1
+        )
+        tasks.config_host_resolvconf_with_master_data(
+            cls.master, cls.replica2
+        )
         # Configure only master and one replica.
         # Replica is configured without CA
         tasks.install_topo(
@@ -785,8 +797,6 @@ class TestReplicaInstallAfterRestore(IntegrationTest):
 
         suffix = ipautil.realm_to_suffix(master.domain.realm)
         suffix = escape_dn_chars(str(suffix))
-        tf = NamedTemporaryFile()
-        ldif_file = tf.name
         entry_ldif = (
             "dn: cn=meTo{hostname},cn=replica,"
             "cn={suffix},"
@@ -802,19 +812,11 @@ class TestReplicaInstallAfterRestore(IntegrationTest):
             "nsds5ReplicaEnabled: off").format(
             hostname=replica1.hostname,
             suffix=suffix)
-        master.put_file_contents(ldif_file, entry_ldif)
-
         # disable replication agreement
-        arg = ['ldapmodify',
-               '-h', master.hostname,
-               '-p', '389', '-D',
-               str(master.config.dirman_dn),  # pylint: disable=no-member
-               '-w', master.config.dirman_password,
-               '-f', ldif_file]
-        master.run_command(arg)
+        tasks.ldapmodify_dm(master, entry_ldif)
 
         # uninstall master.
-        tasks.uninstall_master(master)
+        tasks.uninstall_master(master, clean=False)
 
         # master restore.
         dirman_password = master.config.dirman_password
