@@ -20,20 +20,23 @@
 # FIXME: Pylint errors
 # pylint: disable=no-member
 
+from __future__ import absolute_import
+
 import os
 import re
+import unittest
 
-import nose
+from ipaplatform.constants import constants as platformconstants
 from ipaplatform.paths import paths
 
-from ipatests.test_integration import tasks
+from ipatests.pytest_ipa.integration import tasks
 
 # importing test_trust under different name to avoid nose executing the test
 # base class imported from this module
 from ipatests.test_integration import test_trust as trust_tests
 
 
-class BaseTestLegacyClient(object):
+class BaseTestLegacyClient:
     """
     Tests legacy client support.
     """
@@ -47,6 +50,7 @@ class BaseTestLegacyClient(object):
                     paths.SSSD_CONF]
 
     homedir_template = "/home/{domain}/{username}"
+    default_shell = platformconstants.DEFAULT_SHELL
     required_extra_roles = ()
     optional_extra_roles = ()
 
@@ -58,6 +62,8 @@ class BaseTestLegacyClient(object):
     testuser_gid_regex = None
     subdomain_testuser_uid_regex = None
     subdomain_testuser_gid_regex = None
+    treedomain_testuser_uid_regex = None
+    treedomain_testuser_gid_regex = None
 
     # To allow custom validation dependent on the trust type
     posix_trust = False
@@ -86,8 +92,10 @@ class BaseTestLegacyClient(object):
         self.clear_sssd_caches()
         result = self.legacy_client.run_command(['getent', 'passwd', 'admin'])
 
-        admin_regex = "admin:\*:(\d+):(\d+):"\
-                      "Administrator:/home/admin:/bin/bash"
+        admin_regex = r"admin:\*:(\d+):(\d+):"\
+                      r"Administrator:/home/admin:{}".format(
+                          platformconstants.DEFAULT_ADMIN_SHELL,
+                      )
 
         assert re.search(admin_regex, result.stdout_text)
 
@@ -95,7 +103,7 @@ class BaseTestLegacyClient(object):
         self.clear_sssd_caches()
         result = self.legacy_client.run_command(['getent', 'group', 'admins'])
 
-        admin_group_regex = "admins:\*:(\d+):admin"
+        admin_group_regex = r"admins:\*:(\d+):admin"
 
         assert re.search(admin_group_regex, result.stdout_text)
 
@@ -103,9 +111,9 @@ class BaseTestLegacyClient(object):
         self.clear_sssd_caches()
         result = self.legacy_client.run_command(['id', 'admin'])
 
-        uid_regex = "uid=(\d+)\(admin\)"
-        gid_regex = "gid=(\d+)\(admins\)"
-        groups_regex = "groups=(\d+)\(admins\)"
+        uid_regex = r"uid=(\d+)\(admin\)"
+        gid_regex = r"gid=(\d+)\(admins\)"
+        groups_regex = r"groups=(\d+)\(admins\)"
 
         assert re.search(uid_regex, result.stdout_text)
         assert re.search(gid_regex, result.stdout_text)
@@ -116,14 +124,15 @@ class BaseTestLegacyClient(object):
         testuser = 'testuser@%s' % self.ad.domain.name
         result = self.legacy_client.run_command(['getent', 'passwd', testuser])
 
-        testuser_regex = "testuser@%s:\*:%s:%s:"\
-                         "Test User:%s:/bin/sh"\
+        testuser_regex = r"testuser@%s:\*:%s:%s:"\
+                         r"Test User:%s:%s"\
                          % (re.escape(self.ad.domain.name),
                             self.testuser_uid_regex,
                             self.testuser_gid_regex,
                             self.homedir_template.format(
                                 username='testuser',
-                                domain=re.escape(self.ad.domain.name))
+                                domain=re.escape(self.ad.domain.name)),
+                            self.default_shell,
                             )
 
         assert re.search(testuser_regex, result.stdout_text)
@@ -133,7 +142,7 @@ class BaseTestLegacyClient(object):
         testgroup = 'testgroup@%s' % self.ad.domain.name
         result = self.legacy_client.run_command(['getent', 'group', testgroup])
 
-        testgroup_regex = "%s:\*:%s:" % (testgroup, self.testuser_gid_regex)
+        testgroup_regex = r"%s:\*:%s:" % (testgroup, self.testuser_gid_regex)
         assert re.search(testgroup_regex, result.stdout_text)
 
     def test_id_ad_user(self):
@@ -157,7 +166,7 @@ class BaseTestLegacyClient(object):
 
     def test_login_ipa_user(self):
         if not self.master.transport.file_exists('/usr/bin/sshpass'):
-            raise nose.SkipTest('Package sshpass not available on %s'
+            raise unittest.SkipTest('Package sshpass not available on %s'
                                  % self.master.hostname)
 
         result = self.master.run_command(
@@ -174,7 +183,7 @@ class BaseTestLegacyClient(object):
 
     def test_login_ad_user(self):
         if not self.master.transport.file_exists('/usr/bin/sshpass'):
-            raise nose.SkipTest('Package sshpass not available on %s'
+            raise unittest.SkipTest('Package sshpass not available on %s'
                                  % self.master.hostname)
 
         testuser = 'testuser@%s' % self.ad.domain.name
@@ -191,7 +200,7 @@ class BaseTestLegacyClient(object):
 
     def test_login_disabled_ipa_user(self):
         if not self.master.transport.file_exists('/usr/bin/sshpass'):
-            raise nose.SkipTest('Package sshpass not available on %s'
+            raise unittest.SkipTest('Package sshpass not available on %s'
                                  % self.master.hostname)
 
         self.clear_sssd_caches()
@@ -211,7 +220,7 @@ class BaseTestLegacyClient(object):
 
     def test_login_disabled_ad_user(self):
         if not self.master.transport.file_exists('/usr/bin/sshpass'):
-            raise nose.SkipTest('Package sshpass not available on %s'
+            raise unittest.SkipTest('Package sshpass not available on %s'
                                  % self.master.hostname)
 
         testuser = 'disabledaduser@%s' % self.ad.domain.name
@@ -229,40 +238,41 @@ class BaseTestLegacyClient(object):
 
     def test_getent_subdomain_ad_user(self):
         if not self.ad_subdomain:
-            raise nose.SkipTest('AD for the subdomain is not available.')
+            raise unittest.SkipTest('AD for the subdomain is not available.')
 
         self.clear_sssd_caches()
         testuser = 'subdomaintestuser@%s' % self.ad_subdomain
         result = self.legacy_client.run_command(['getent', 'passwd', testuser])
 
-        testuser_regex = "subdomaintestuser@%s:\*:%s:%s:"\
-                         "Subdomaintest User:%s:"\
-                         "/bin/sh"\
+        testuser_regex = r"subdomaintestuser@%s:\*:%s:%s:"\
+                         r"Subdomaintest User:%s:"\
+                         r"%s"\
                          % (re.escape(self.ad_subdomain),
                             self.subdomain_testuser_uid_regex,
                             self.subdomain_testuser_gid_regex,
                             self.homedir_template.format(
                                 username='subdomaintestuser',
-                                domain=re.escape(self.ad_subdomain))
+                                domain=re.escape(self.ad_subdomain)),
+                            self.default_shell,
                             )
 
         assert re.search(testuser_regex, result.stdout_text)
 
     def test_getent_subdomain_ad_group(self):
         if not self.ad_subdomain:
-            raise nose.SkipTest('AD for the subdomain is not available.')
+            raise unittest.SkipTest('AD for the subdomain is not available.')
 
         self.clear_sssd_caches()
         testgroup = 'subdomaintestgroup@%s' % self.ad_subdomain
         result = self.legacy_client.run_command(['getent', 'group', testgroup])
 
-        testgroup_stdout = "%s:\*:%s:" % (testgroup,
+        testgroup_stdout = r"%s:\*:%s:" % (testgroup,
                                           self.subdomain_testuser_gid_regex)
         assert re.search(testgroup_stdout, result.stdout_text)
 
     def test_id_subdomain_ad_user(self):
         if not self.ad_subdomain:
-            raise nose.SkipTest('AD for the subdomain is not available.')
+            raise unittest.SkipTest('AD for the subdomain is not available.')
 
         self.clear_sssd_caches()
         testuser = 'subdomaintestuser@%s' % self.ad_subdomain
@@ -287,10 +297,10 @@ class BaseTestLegacyClient(object):
 
     def test_login_subdomain_ad_user(self):
         if not self.ad_subdomain:
-            raise nose.SkipTest('AD for the subdomain is not available.')
+            raise unittest.SkipTest('AD for the subdomain is not available.')
 
         if not self.master.transport.file_exists('/usr/bin/sshpass'):
-            raise nose.SkipTest('Package sshpass not available on %s'
+            raise unittest.SkipTest('Package sshpass not available on %s'
                                  % self.master.hostname)
 
         testuser = 'subdomaintestuser@%s' % self.ad_subdomain
@@ -307,10 +317,10 @@ class BaseTestLegacyClient(object):
 
     def test_login_disabled_subdomain_ad_user(self):
         if not self.ad_subdomain:
-            raise nose.SkipTest('AD for the subdomain is not available.')
+            raise unittest.SkipTest('AD for the subdomain is not available.')
 
         if not self.master.transport.file_exists('/usr/bin/sshpass'):
-            raise nose.SkipTest('Package sshpass not available on %s'
+            raise unittest.SkipTest('Package sshpass not available on %s'
                                  % self.master.hostname)
 
         testuser = 'subdomaindisabledaduser@%s' % self.ad_subdomain
@@ -326,6 +336,85 @@ class BaseTestLegacyClient(object):
 
         assert result.returncode != 0
 
+    def test_getent_treedomain_ad_user(self):
+        if not self.ad_treedomain:
+            raise unittest.SkipTest('AD tree root domain is not available.')
+
+        self.clear_sssd_caches()
+        testuser = 'treetestuser@{0}'.format(self.ad_treedomain)
+        result = self.legacy_client.run_command(['getent', 'passwd', testuser])
+
+        testuser_regex = (r"treetestuser@{0}:\*:{1}:{2}:TreeTest User:"
+                          r"/home/{0}/treetestuser:{3}".format(
+                              re.escape(self.ad_treedomain),
+                              self.treedomain_testuser_uid_regex,
+                              self.treedomain_testuser_gid_regex,
+                              self.default_shell,
+                          ))
+
+        assert re.search(testuser_regex, result.stdout_text)
+
+    def test_getent_treedomain_ad_group(self):
+        if not self.ad_treedomain:
+            raise unittest.SkipTest('AD tree root domain is not available')
+
+        self.clear_sssd_caches()
+        testgroup = 'treetestgroup@{0}'.format(self.ad_treedomain)
+        result = self.legacy_client.run_command(['getent', 'group', testgroup])
+
+        testgroup_stdout = r"{0}:\*:{1}:".format(
+                           testgroup, self.treedomain_testuser_gid_regex)
+
+        assert re.search(testgroup_stdout, result.stdout_text)
+
+    def test_id_treedomain_ad_user(self):
+        if not self.ad_treedomain:
+            raise unittest.SkipTest('AD tree root domain is not available')
+
+        self.clear_sssd_caches()
+
+        testuser = 'treetestuser@{0}'.format(self.ad_treedomain)
+        testgroup = 'treetestgroup@{0}'.format(self.ad_treedomain)
+
+        result = self.legacy_client.run_command(['id', testuser])
+
+        # Only for POSIX trust testing does the testuser belong to the
+        # testgroup
+
+        group_name = '\({}\)'.format(testgroup) if self.posix_trust else ''
+
+        uid_regex = "uid={0}\({1}\)".format(
+                    self.treedomain_testuser_uid_regex, testuser)
+
+        gid_regex = "gid={0}{1}".format(
+                    self.treedomain_testuser_gid_regex, group_name)
+
+        group_regex = "groups={0}{1}".format(
+                      self.treedomain_testuser_gid_regex, group_name)
+
+        assert re.search(uid_regex, result.stdout_text)
+        assert re.search(gid_regex, result.stdout_text)
+        assert re.search(group_regex, result.stdout_text)
+
+    def test_login_treedomain_ad_user(self):
+        if not self.ad_treedomain:
+            raise unittest.SkipTest('AD tree root domain is not available.')
+
+        if not self.master.transport.file_exists('/usr/bin/sshpass'):
+            raise unittest.SkipTest(
+                'Package sshpass not available on {}'.format(
+                    self.master.hostname)
+            )
+
+        result = self.master.run_command(
+            'sshpass -p {0} ssh -o StrictHostKeyChecking=no '
+            '-l admin {1} "echo test"'.format(
+                self.legacy_client.config.admin_password,
+                self.legacy_client.external_hostname))
+
+        assert "test" in result.stdout_text
+
+
     @classmethod
     def install(cls, mh):
         super(BaseTestLegacyClient, cls).install(mh)
@@ -354,10 +443,18 @@ class BaseTestLegacyClient(object):
         try:
             child_ad = cls.host_by_role(cls.optional_extra_roles[0])
             cls.ad_subdomain = '.'.join(
-                                   child_ad.hostname.split('.')[1:])
+                child_ad.hostname.split('.')[1:])
         except LookupError:
             cls.ad_subdomain = None
 
+        # Determine whether the tree domain AD is available
+        try:
+            cls.tree_ad = cls.host_by_role(cls.optional_extra_roles[1])
+            cls.ad_treedomain = '.'.join(
+                cls.tree_ad.hostname.split('.')[1:])
+        except LookupError:
+            cls.ad_treedomain = None
+
         tasks.apply_common_fixes(cls.legacy_client)
 
         for f in cls.backup_files:
@@ -381,14 +478,14 @@ class BaseTestLegacyClient(object):
 
 # Base classes with attributes that are specific for each legacy client test
 
-class BaseTestLegacySSSDBefore19RedHat(object):
+class BaseTestLegacySSSDBefore19RedHat:
 
     advice_id = 'config-redhat-sssd-before-1-9'
     required_extra_roles = ['legacy_client_sssd_redhat']
     optional_extra_roles = ['ad_subdomain', 'ad_treedomain']
 
 
-class BaseTestLegacyNssPamLdapdRedHat(object):
+class BaseTestLegacyNssPamLdapdRedHat:
 
     advice_id = 'config-redhat-nss-pam-ldapd'
     required_extra_roles = ['legacy_client_nss_pam_ldapd_redhat']
@@ -398,7 +495,7 @@ class BaseTestLegacyNssPamLdapdRedHat(object):
         tasks.clear_sssd_cache(self.master)
 
 
-class BaseTestLegacyNssLdapRedHat(object):
+class BaseTestLegacyNssLdapRedHat:
 
     advice_id = 'config-redhat-nss-ldap'
     required_extra_roles = ['legacy_client_nss_ldap_redhat']
@@ -411,32 +508,36 @@ class BaseTestLegacyNssLdapRedHat(object):
 # Base classes that join legacy client specific steps with steps required
 # to setup IPA with trust (both with and without using the POSIX attributes)
 
-class BaseTestLegacyClientPosix(BaseTestLegacyClient,
-                                trust_tests.TestEnforcedPosixADTrust):
+class BaseTestLegacyClientPosix(trust_tests.BaseTestTrust,
+                                BaseTestLegacyClient):
 
     testuser_uid_regex = '10042'
     testuser_gid_regex = '10047'
     subdomain_testuser_uid_regex = '10142'
     subdomain_testuser_gid_regex = '10147'
+    treedomain_testuser_uid_regex = '10242'
+    treedomain_testuser_gid_regex = '10247'
     posix_trust = True
 
     def test_remove_trust_with_posix_attributes(self):
         pass
 
 
-class BaseTestLegacyClientNonPosix(BaseTestLegacyClient,
-                                   trust_tests.TestBasicADTrust):
+class BaseTestLegacyClientNonPosix(trust_tests.BaseTestTrust,
+                                   BaseTestLegacyClient):
 
-    testuser_uid_regex = '(?!10042)(\d+)'
-    testuser_gid_regex = '(?!10047)(\d+)'
-    subdomain_testuser_uid_regex = '(?!10142)(\d+)'
-    subdomain_testuser_gid_regex = '(?!10147)(\d+)'
+    testuser_uid_regex = r'(?!10042)(\d+)'
+    testuser_gid_regex = r'(?!10047)(\d+)'
+    subdomain_testuser_uid_regex = r'(?!10142)(\d+)'
+    subdomain_testuser_gid_regex = r'(?!10147)(\d+)'
+    treedomain_testuser_uid_regex = r'(?!10242)(\d+)'
+    treedomain_testuser_gid_regex = r'(?!10247)(\d+)'
 
     def test_remove_nonposix_trust(self):
         pass
 
 
-class BaseTestSSSDMixin(object):
+class BaseTestSSSDMixin:
 
     def test_apply_advice(self):
         super(BaseTestSSSDMixin, self).test_apply_advice()
