@@ -25,18 +25,18 @@
 
 # The DM password needs to be set in ~/.ipa/.dmpw
 
+from __future__ import absolute_import
+
 import os
 import sys
+import unittest
 
 import pytest
-import nose
-from nose.tools import assert_raises  # pylint: disable=E0611
-import nss.nss as nss
 import six
 
-from ipaserver.plugins.ldap2 import ldap2
-from ipalib import api, x509, create_api, errors
-from ipapython import ipautil
+from ipaplatform.paths import paths
+from ipaserver.plugins.ldap2 import ldap2, AUTOBIND_DISABLED
+from ipalib import api, create_api, errors
 from ipapython.dn import DN
 
 if six.PY3:
@@ -44,15 +44,15 @@ if six.PY3:
 
 
 @pytest.mark.tier0
-class test_ldap(object):
+@pytest.mark.needs_ipaapi
+class test_ldap:
     """
     Test various LDAP client bind methods.
     """
 
     def setup(self):
         self.conn = None
-        self.ldapuri = 'ldap://%s' % ipautil.format_netloc(api.env.host)
-        nss.nss_init_nodb()
+        self.ldapuri = api.env.ldap_uri
         self.dn = DN(('krbprincipalname','ldap/%s@%s' % (api.env.host, api.env.realm)),
                      ('cn','services'),('cn','accounts'),api.env.basedn)
 
@@ -64,8 +64,8 @@ class test_ldap(object):
         """
         Test an anonymous LDAP bind using ldap2
         """
-        self.conn = ldap2(api, ldap_uri=self.ldapuri)
-        self.conn.connect()
+        self.conn = ldap2(api)
+        self.conn.connect(autobind=AUTOBIND_DISABLED)
         dn = api.env.basedn
         entry_attrs = self.conn.get_entry(dn, ['associateddomain'])
         domain = entry_attrs.single_value['associateddomain']
@@ -75,32 +75,29 @@ class test_ldap(object):
         """
         Test a GSSAPI LDAP bind using ldap2
         """
-        self.conn = ldap2(api, ldap_uri=self.ldapuri)
-        self.conn.connect()
+        self.conn = ldap2(api)
+        self.conn.connect(autobind=AUTOBIND_DISABLED)
         entry_attrs = self.conn.get_entry(self.dn, ['usercertificate'])
-        cert = entry_attrs.get('usercertificate')
-        cert = cert[0]
-        serial = unicode(x509.get_serial_number(cert, x509.DER))
-        assert serial is not None
+        cert = entry_attrs.get('usercertificate')[0]
+        assert cert.serial_number is not None
 
     def test_simple(self):
         """
         Test a simple LDAP bind using ldap2
         """
         pwfile = api.env.dot_ipa + os.sep + ".dmpw"
-        if ipautil.file_exists(pwfile):
-            fp = open(pwfile, "r")
-            dm_password = fp.read().rstrip()
-            fp.close()
+        if os.path.isfile(pwfile):
+            with open(pwfile, "r") as fp:
+                dm_password = fp.read().rstrip()
         else:
-            raise nose.SkipTest("No directory manager password in %s" % pwfile)
-        self.conn = ldap2(api, ldap_uri=self.ldapuri)
+            raise unittest.SkipTest(
+                "No directory manager password in %s" % pwfile
+            )
+        self.conn = ldap2(api)
         self.conn.connect(bind_dn=DN(('cn', 'directory manager')), bind_pw=dm_password)
         entry_attrs = self.conn.get_entry(self.dn, ['usercertificate'])
-        cert = entry_attrs.get('usercertificate')
-        cert = cert[0]
-        serial = unicode(x509.get_serial_number(cert, x509.DER))
-        assert serial is not None
+        cert = entry_attrs.get('usercertificate')[0]
+        assert cert.serial_number is not None
 
     def test_Backend(self):
         """
@@ -111,44 +108,41 @@ class test_ldap(object):
         # a client-only api. Then we register in the commands and objects
         # we need for the test.
         myapi = create_api(mode=None)
-        myapi.bootstrap(context='cli', in_server=True)
+        myapi.bootstrap(context='cli', in_server=True, confdir=paths.ETC_IPA)
         myapi.finalize()
 
         pwfile = api.env.dot_ipa + os.sep + ".dmpw"
-        if ipautil.file_exists(pwfile):
-            fp = open(pwfile, "r")
-            dm_password = fp.read().rstrip()
-            fp.close()
+        if os.path.isfile(pwfile):
+            with open(pwfile, "r") as fp:
+                dm_password = fp.read().rstrip()
         else:
-            raise nose.SkipTest("No directory manager password in %s" % pwfile)
+            raise unittest.SkipTest(
+                "No directory manager password in %s" % pwfile
+            )
         myapi.Backend.ldap2.connect(bind_dn=DN(('cn', 'Directory Manager')), bind_pw=dm_password)
 
         result = myapi.Command['service_show']('ldap/%s@%s' %  (api.env.host, api.env.realm,))
         entry_attrs = result['result']
-        cert = entry_attrs.get('usercertificate')
-        cert = cert[0]
-        serial = unicode(x509.get_serial_number(cert, x509.DER))
-        assert serial is not None
+        cert = entry_attrs.get('usercertificate')[0]
+        assert cert.serial_number is not None
 
     def test_autobind(self):
         """
         Test an autobind LDAP bind using ldap2
         """
-        ldapuri = 'ldapi://%%2fvar%%2frun%%2fslapd-%s.socket' % api.env.realm.replace('.','-')
-        self.conn = ldap2(api, ldap_uri=ldapuri)
+        self.conn = ldap2(api)
         try:
             self.conn.connect(autobind=True)
         except errors.ACIError:
-            raise nose.SkipTest("Only executed as root")
+            raise unittest.SkipTest("Only executed as root")
         entry_attrs = self.conn.get_entry(self.dn, ['usercertificate'])
-        cert = entry_attrs.get('usercertificate')
-        cert = cert[0]
-        serial = unicode(x509.get_serial_number(cert, x509.DER))
-        assert serial is not None
+        cert = entry_attrs.get('usercertificate')[0]
+        assert cert.serial_number is not None
 
 
 @pytest.mark.tier0
-class test_LDAPEntry(object):
+@pytest.mark.needs_ipaapi
+class test_LDAPEntry:
     """
     Test the LDAPEntry class
     """
@@ -158,9 +152,9 @@ class test_LDAPEntry(object):
     dn2 = DN(('cn', cn2[0]))
 
     def setup(self):
-        self.ldapuri = 'ldap://%s' % ipautil.format_netloc(api.env.host)
-        self.conn = ldap2(api, ldap_uri=self.ldapuri)
-        self.conn.connect()
+        self.ldapuri = api.env.ldap_uri
+        self.conn = ldap2(api)
+        self.conn.connect(autobind=AUTOBIND_DISABLED)
 
         self.entry = self.conn.make_entry(self.dn1, cn=self.cn1)
 
@@ -240,7 +234,7 @@ class test_LDAPEntry(object):
         assert e.pop('cn') == self.cn1
         assert 'cn' not in e
         assert e.pop('cn', 'default') is 'default'
-        with assert_raises(KeyError):
+        with pytest.raises(KeyError):
             e.pop('cn')
 
     def test_clear(self):
@@ -323,3 +317,21 @@ class test_LDAPEntry(object):
 
         e.raw['test'].append(b'second')
         assert e['test'] == ['not list', u'second']
+
+    def test_modlist_with_varying_encodings(self):
+        """
+        Test modlist is correct when only encoding of new value differs
+
+        See: https://bugzilla.redhat.com/show_bug.cgi?id=1658302
+        """
+        dn_ipa_encoded = b'O=Red Hat\\, Inc.'
+        dn_389ds_encoded = b'O=Red Hat\\2C Inc.'
+        entry = self.entry
+        entry.raw['distinguishedName'] = [dn_389ds_encoded]
+        # This is to make entry believe that that value was part of the
+        # original data we received from LDAP
+        entry.reset_modlist()
+        entry['distinguishedName'] = [entry['distinguishedName'][0]]
+        assert entry.generate_modlist() == [
+            (1, 'distinguishedName', [dn_389ds_encoded]),
+            (0, 'distinguishedName', [dn_ipa_encoded])]
