@@ -49,7 +49,7 @@ else:
 pattern_type = type(re.compile(""))
 
 
-class Prop(object):
+class Prop:
     def __init__(self, *ops):
         self.__ops = frozenset(ops)
         self.__prop = 'prop value'
@@ -72,7 +72,7 @@ class Prop(object):
     prop = property(__get_prop, __set_prop, __del_prop)
 
 
-class test_Fuzzy(object):
+class test_Fuzzy:
     klass = util.Fuzzy
 
     def test_init(self):
@@ -152,10 +152,10 @@ class test_Fuzzy(object):
         assert (None == self.klass()) is True
 
 
-def test_assert_deepequal():
+def test_assert_deepequal(pytestconfig):
     f = util.assert_deepequal
     try:  # pylint: disable=no-member
-        pretty = pytest.config.getoption("pretty_print")
+        pretty = pytestconfig.getoption("pretty_print")
     except (AttributeError, ValueError):
         pretty = False
 
