@@ -24,6 +24,7 @@ Trust tests
 from ipatests.test_webui.ui_driver import UI_driver
 from ipatests.test_webui.ui_driver import screenshot
 from ipatests.test_webui.task_range import range_tasks
+import pytest
 
 ENTITY = 'trust'
 CONFIG_ENTITY = 'trustconfig'
@@ -41,6 +42,7 @@ CONFIG_DATA2 = {
 }
 
 
+@pytest.mark.tier1
 class trust_tasks(UI_driver):
 
     def get_data(self, add_data=None):
@@ -93,11 +95,13 @@ class trust_tasks(UI_driver):
         return domain.upper() + '_id_range'
 
 
+@pytest.mark.tier1
 class test_trust(trust_tasks):
 
-    def __init__(self, *args, **kwargs):
-        super(test_trust, self).__init__(args, kwargs)
+    request_timeout = 120
 
+    def setup(self, *args, **kwargs):
+        super(test_trust, self).setup(*args, **kwargs)
         if not self.has_trusts():
             self.skip('Trusts not configured')
 
@@ -121,11 +125,13 @@ class test_trust(trust_tasks):
 
         self.init_app()
 
-        r_tasks = range_tasks(self.driver, self.config)
+        r_tasks = range_tasks()
+        r_tasks.driver = self.driver
+        r_tasks.config = self.config
         r_tasks.get_shifts()
-        range_add = r_tasks.get_add_data('')
-        base_id = range_add[2][2]
-        range_size = range_add[3][2]
+        range_form = r_tasks.get_add_form_data('')
+        base_id = range_form.base_id
+        range_size = range_form.size
         range_pkey = self.get_range_name()
         column = 'iparangetype'
 
@@ -139,7 +145,6 @@ class test_trust(trust_tasks):
         self.assert_record_value('Active Directory domain range', range_pkey, column)
         self.delete_record(range_pkey)
 
-        self.request_timeout = 60
         add = self.get_add_data('ipa-ad-trust-posix', base_id, range_size)
         data = self.get_data(add_data=add)
         self.add_record(ENTITY, data, delete=True)
